#############################################################################
##
## %File: assayplots.R
## %Plug-in: Visualization: Array plots 
## %Version: R2:1.0
##
## %Author: Adam Ameur, Linnaeus Centre for Bioinformatics
## %Created: 29/3 2004
## %Modified: 9/6 2006
##
## %Description: The plug-in plots a number of values (foreground intensities,
##  background intensities, log2-ratios and flags) onto representations of
##  the array slides. A number of additional plots may optionally be selected.
##  The plots may be useful for assessing the microarray data quality and to
##  detect systematic variation.
##
## %Output: A number of plots are produced and assembeled on a .html page.
##  1. Intensity plots: ch1 and ch2 intensities plotted onto array
##  2. Background plots: ch1 and ch2 background intensities plotted onto array
##  3. M-value or ratio plots: log2(int1/int2) or int1/int2 plotted onto array
##  4. Flag plots: Flag values plotted onto array
##  5. SNR Histograms
##  6. MA-plots: with or without loess curves for each print-tip group
##  7. Box plots for each print tip group
##  8. Density plots: Distribution of M-values
##  9. Overall plot
##
## %Parameters:
##  - Selection of plots: There options are reduced (1,3,6,8 and 9), normal
##    (all except 5) and extended (all plots). The reduced option may be useful
##    in later stages of analysis, for example after the data has been merged
##    and no longer agree with the original array layout.
##  - Ratio type: M-values (log2-ratios) or ratios can be plotted in plot 3.
##  - Specify array layout dimensions: Should be selected only if array layout
##    dimensions have not been put into the database at an earlier stage.
##  - Grid rows/columns, Spot rows/columns per print-tip group: Information
##    about the array layout. Must be provided when the specify array layout
##    dimensions option is selected.
##
## %Screen shots: arrayplot1.png (500), arrayplot2.png (250)
##
################################################################################

## Ignore warnings
options(warn = -1)

source("../../../../configR2.R")

## Include functions for BASEfile read/write
source(paste(config["plugins_R_dir"],"/common.R",sep=""))
## Include help functions for array layout
source(paste(config["plugins_R_dir"],"/layout.R",sep=""))
## Include help functions for plots
source(paste(config["plugins_R_dir"],"/assayplots.inc.R",sep=""))

## Load required libraries
library(sma)
library(limma)

## Load data and parameters
pars<-read.parameters()
data<-read.data()

## Set up directory for the job where all plots are stored
picsDir<-"pics/"
dir.create(picsDir)

## Load grid layouts on the arrays
layouts<-read.layout()

plotGroup <- pars$plotGroup            # 1:Reduced, 2:Normal, 3:Extended
ratioType <- pars$ratioType            # 1:Log2-ratio, 2:Ratio

M<-list()
nrAssays<-length(data)

## #########################################
##
## Set height and width of plots. 
##
## ##########################################

## Height and width of array plots in inches
heightInches<-12 
widthInches<-12  

if (plotGroup == 1) {
  heightInches<-10 
}

if (plotGroup == 3) {
  heightInches<-14 
}

## 72 dots per inches is default
dpi<-72

## Height and width of array plots in pixels
height<-heightInches*dpi
width<-widthInches*dpi   

## Height and width of thumbnail density plot in pixels
heightThumb<-min(max((height-50)/nrAssays, 30), 120)
widthThumb<-heightThumb

## Height and width of thumbnail density plot in inches
heightThumbInches<-heightThumb/dpi
widthThumbInches<-heightThumbInches

debug.output("Data and parameters loaded")

for (i in 1:nrAssays){

  ## Create array layout
  layout <- create.layout(pars, layouts[[i]], data[[i]]$assayId)
  
  ## Fill missing positions with NA values
  assayData <- fill.missing.positions(data[[i]], layout, extended = TRUE)
  
  RG<-list(R=as.matrix(assayData$intensity1), G=as.matrix(assayData$intensity2))
  MA<-MA.RG(RG)
  
  M[[i]]<-MA$M

  signalsR<-NULL
  if (ratioType == 2)
    signalsR <- assayData$intensity1 / assayData$intensity2
  
  ## ###########################
  ##
  ## Create plots for the array
  ##		
  ## ###########################

  ## Prepare layout for plots.
  coordinates<-calculate.layout.id(layout)

  xCoord <- coordinates[,1]
  yCoord <- coordinates[,2]
    
  dataOrder <- order(xCoord)
  
  numx=layout$ngrid.r*layout$nspot.r
  numy=layout$ngrid.c*layout$nspot.c

  sortData <- NULL
  if (ratioType == 1)
    sortData <- cbind(assayData$intensity1,assayData$intensity2,assayData$BCh1,assayData$BCh2,assayData$flags, M[[i]])
  else
    sortData <- cbind(assayData$intensity1,assayData$intensity2,assayData$BCh1,assayData$BCh2,assayData$flags, M[[i]], signalsR)

  sortData <- sortData[dataOrder, ]

  signalsInt1<-matrix(sortData[ ,1],numx,numy)
  signalsInt2<-matrix(sortData[ ,2],numx,numy)
  signalsBCh1<-matrix(sortData[ ,3],numx,numy)
  signalsBCh2<-matrix(sortData[ ,4],numx,numy)
  signalsFlags<-matrix(sortData[ ,5],numx,numy)
  signalsM<-matrix(sortData[ ,6],numx,numy)
  
  if (ratioType == 2)
    signalsR<-matrix(sortData[ ,7],numx,numy)
  
  x<-seq(1,numx)
  y<-seq(1,numy)
  
  imgFileName=paste(picsDir,"imgplot",i,".png",sep="")

  ## Set boundary values for intensities and background. Boundary
  ## values are used for creating scales later used in the plots.
  bound=quantile(append(assayData$intensity1,assayData$intensity2),probs=c(0.05,0.95),na.rm=TRUE)
  boundBg=quantile(append(assayData$BCh1,assayData$BCh2),probs=c(0.05,0.95),na.rm=TRUE)
  boundFlags=c(min(assayData$flags, na.rm=TRUE),max(assayData$flags, na.rm=TRUE))
  
  ## Set the layout of the array plot (4x4 image plots with legends)
  if (plotGroup == 1) {
    bitmap(file=imgFileName, type = "png16m", height = heightInches, width = widthInches)
    layout(matrix(c(1,2,3,4,5,6,8,9,7,7,9,9), 3, 4, byrow=TRUE),widths=c(15,1,15,1),heights=c(10,10,15))
  }
  
  else if (plotGroup == 2) {
    bitmap(file=imgFileName, type = "png16m", height = heightInches, width = widthInches)
    layout(matrix(c(1,2,3,4,5,6,7,8,9,10,11,12,13,15,14,15), 4, 4, byrow=TRUE),widths=c(15,1,15,1),heights=c(10,10,10,18))
  }

  else {
    bitmap(file=imgFileName, type = "png16m", height = heightInches, width = widthInches)
    layout(matrix(c(1,2,3,4,5,6,7,8,9,10,11,12,13,17,14,17,15,17,16,17), 5, 4, byrow=TRUE),widths=c(15,1,15,1),heights=c(10,10,10,14,18))
  }
  
  par(oma=c(0,0,2,0))

  ## Plot intensities
  arrayplot.image(x, y, signalsInt1, layout, bound, yellow.red(101), "Intensity 1")
  arrayplot.image(x, y, signalsInt2, layout, bound, yellow.red(101), "Intensity 2")

  ## Plot backgrounds (not for reduced plots)
  if (plotGroup != 1) {
    arrayplot.image(x, y, signalsBCh1, layout, boundBg, yellow.red(101), "Background 1")
    arrayplot.image(x, y, signalsBCh2, layout, boundBg, yellow.red(101), "Background 2")
  }

  if (ratioType == 1) {
    ## Plot M-values
    arrayplot.image(x, y, signalsM, layout, c(-3,3), green.red(101), "Log2-ratio")
  }
  else {
    ## Plot Ratios
    arrayplot.image(x, y, signalsR, layout, c(0,3), green.red(101), "Ratio")
  }

  ## Plot flags (not for reduced plots)
  if (plotGroup != 1) {
    arrayplot.image(x, y, signalsFlags, layout, boundFlags, black.blue(9), "Flags")
  }

  par(mar=c(3,2,3,1))

  ## Make SNR plots for both channels (only for plotGroup == 3)
  if (plotGroup == 3) {
    SNR.Ch1 <- (assayData$intensity1 - assayData$BCh1) / assayData$BCh1SD
    SNR.Ch1 <- SNR.Ch1[which(SNR.Ch1 <= 50)]
    min.snr <- floor(min(SNR.Ch1))
    hist(SNR.Ch1,breaks=min.snr:50,col="grey",
         main=paste("SNR 1 Histogram [",min.snr,",50]",sep=""))
    box()

    SNR.Ch2 <- (assayData$intensity2 - assayData$BCh2) / assayData$BCh2SD
    SNR.Ch2 <- SNR.Ch2[which(SNR.Ch2 <= 50)]
    min.snr <- floor(min(SNR.Ch2))
    hist(SNR.Ch2,breaks=min.snr:50,col="grey",
         main=paste("SNR 2 Histogram [",min.snr,",50]",sep=""))
    box()
  }

  ## Make MA-plot (only for reduced plots)
  if (plotGroup == 1) {
    plotMA(MA, main="MA-plot")
  }
  ## Make MA-plot with loess curves for print tip groups and 
  ## boxplots for each print tip group (not for reduced plots)
  else{
    errMsg <- try(plot.print.tip.lowess(RG,layout,pch=16,cex=0.2,image=1, main="MA-plot with lowess curves"), silent = TRUE)
    if(!is.null(errMsg)){
      warning("assayplots.R", "lowess lines for print-tip group(s) might be missing in MA-plot.")
    }

    plot.scale.box(as.vector(M[[i]]),layout,col=rainbow(layout$ngrid.r*layout$ngrid.c), main="Boxplot of print-tip groups")
    abline(0,0)
  }

  ## Write assay name as title for the plot
  mtext(data[[i]]$assayName, outer=T, cex=1.3, col="red")

  dev.off()

  ## ###################################
  ##
  ## Create density plots for the array
  ##
  ## ###################################

  M.values<-M[[i]]
  M.values<-M.values[M.values>-Inf & M.values<Inf]
  
  ## Create thumbnail density plot 
  bitmap(file=paste(picsDir,"thumb_density",i,".png",sep=""), type = "png16m", height = heightThumbInches, width = widthThumbInches)
  par(mar=c(1,1,1,1)+0.1)
  plot(density(M.values,bw=0.17,na.rm=TRUE), xlim=range(-5,5), type = "l", col = "red",  frame.plot=FALSE, xlab = "M-value", ylab = "", main="")
  abline(0,9999,0,col="blue")
  dev.off()
  
  ## Create full size density plot
  bitmap(file=paste(picsDir,"density",i,".png",sep=""), type = "png16m", height = 5, width = 5, res = 72)
  par(mar=c(5,4,4,2)+0.1)
  plot(density(M.values,bw=0.17,na.rm=TRUE),xlim=range(-5,5),type = "l", col = "red",  xlab = "M-value", ylab = "",  cex.axis = 1.5, cex.lab=1.5, main= data[[i]]$assayName, cex.main=2)
  abline(0,9999,0,col="blue")
  dev.off()

  debug.output(paste("Produced plots for array",i))
}

## #####################
##
## Create summary plot 
##
## #####################

overallFile<-"overall.png"

bitmap(file=paste(picsDir,overallFile,sep=""), type = "png16m", height = 5, width = 10, res = 72)
boxplot(M,col=rainbow(nrAssays))
abline(0,0,col="blue")
dev.off()

debug.output("Produced summary plots")

## ###########################
##
## Create HTML result page 
##
## ###########################

source(paste(config["plugins_R_dir"],"/rhtml.R", sep=""))
resFile<-"Results.html"

blankname<-paste(picsDir,"blank.png", sep="")

get.blank(fname = blankname, width = widthInches, height = heightInches)

cat("<html>\n", file=resFile)
cat(styleStr, file=resFile, append=TRUE)
cat("<h2>Plots for each array</h2><p>", file=resFile, append=TRUE)

## Generate html code for density thumbnail plots 
densityData<-data.frame(make.pic.link(paste("density",1:nrAssays,".png",sep=""), thumbPrefix="thumb_", path=picsDir, height=heightThumb, width=widthThumb))

## Create the result table with all plots of individual arrays
resultTable<-pic.table(header=data.frame("Array", "Density plot"),pics=paste(picsDir,"imgplot",1:nrAssays,".png",sep=""), data=densityData, width = width,height = height, blankname=blankname)

cat(resultTable, file=resFile, append=TRUE)
cat("<p>", file=resFile, append=TRUE)

## Display overall box plot
cat("<p><h2>Overall distributions of M-values for all arrays</h2><p>", file=resFile, append=TRUE)
cat(paste("<img src=\"",picsDir,overallFile,"\">", sep=""), file=resFile, append=TRUE)
cat("<p><hr><p>", file=resFile, append=TRUE)

cat("</body></html>", file=resFile, append=TRUE)

debug.output("Finished")

clean.up()
