######################################################################
##
## File: assayplots_inc.R
## Version: R2:1.0
##
## Author: Adam Ameur, Linnaeus Centre for Bioinformatics, 29/3 2004
## Latest modified: 9/6 2006
##
## Description: Help functions for assayplots.R for plotting values
## for each spot onto a 'physical' array.
##
#######################################################################

## This function calculates the layout coordinates when plotting
## values for each spot on a 'physical' array. 
calculate.layout.id<-function(layout){
  
  ## Read array layout 
  block.row <- layout$ngrid.r
  block.column <- layout$ngrid.c
  rows <- layout$nspot.r
  cols <- layout$nspot.c
  
  spots <- rows*cols
  blocks <-  block.row*block.column
  
  ## Calcualte location id-vectors
  array.column<-rep(sort(rep(1:block.column,spots)),block.row)
  column<-rep(1:cols,blocks*rows)
  
  array.row<-sort(rep(1:block.row,spots*block.column))
  row<-rep(sort(rep(1:rows,cols)),blocks)
  
  xCoord<-column+cols*(array.column-1)
  yCoord<-rev(row+rows*(array.row-1))

  return(cbind(xCoord, yCoord))
}



## Plots a value for each spot on a 'physical' array
arrayplot.image<-function(x,y,signals,layout,range,colors,text){
  
  ## Plots an image legend for the images produced by arrayplot.image
  image.legend<-function(colors, signal.range, n){

    par(mar=c(2,0,3,1))
    
    signal.sequence <- seq(from = signal.range[1], to = signal.range[2], length = n)
    legend.matrix <- t(as.matrix(signal.sequence))
    
    image(legend.matrix,col=colors,axes=F)

    nr.levels<-3
    legend.text<-as.character(round(seq(signal.range[1],signal.range[2],length=nr.levels)))
	
    ## sets text
    for(i in 1:nr.levels)
      mtext(legend.text[i],side = 2,line=0.2,cex=0.6, at = par()$usr[4]*(i-1)/(nr.levels-1),adj=i/nr.levels)
    
    invisible()
  }

  low=range[1]
  hi=range[2]
  
  signals[which(signals>hi)] <- hi
  signals[which(signals<low)] <- low
  
  par(mar=c(1,2,2,1))
  
  image(x, y, signals,zlim=range,axes=F,col=colors, xaxs="r", yaxs="r")

  xLinePos <- seq(0.5,layout$ngrid.r*layout$nspot.r,layout$nspot.r)
  yLinePos <- seq(0.5,layout$ngrid.c*layout$nspot.c,layout$nspot.c)
  
  abline(v = xLinePos, col = "white")
  abline(h = yLinePos, col = "white")

  mtext(text,side=3,line=0.2, cex = 0.7)
  
  mtext(1:length(xLinePos),side=1,at=xLinePos+(layout$nspot.r/2),las=1,line=0, cex = 0.7)
  mtext(1:length(yLinePos),side=2,at=yLinePos+(layout$nspot.r/2),las=1,line=0, cex = 0.7)
  
  image.legend(colors, range, 50)

  invisible()
}


## ###################################
##
## Functions for creating color maps 
##
## ###################################

## n = number of "levels" (color shades)

## Creates a colormap from yellow to red, with one shade of purple added to it.
yellow.red<-function(n){

  ## Create shades from yellow to red
  red <- seq(from = 255/255,to = 255/255, length=n-1)
  green <-seq(from = 255/255,to = 0/255, length=n-1) 
  blue <- seq(from = 0/255,to = 0/255, length=n-1)

  ## Add one shade of purple (at the last position).
  red <- c(red, 155/255)
  green <- c(green, 48/255)
  blue <- c(blue, 200/255)
  
  ## Returns the colormap
  rgb(red, green, blue)
	
}

## Creates a colormap from green to red.
green.red<-function(n){
  
  ## Create shades from green to red
  red <- seq(from = 0/255,to = 255/255, length=n-1)
  green <-seq(from = 255/255,to = 0/255, length=n-1) 
  blue <- seq(from = 0/255,to = 0/255, length=n-1)

  ## Return the colormap
  rgb(red, green, blue)
	
}

## Creates a colormap from black to blue.
black.blue<-function(n){
  
  red1 <- seq(from = 0/255,to = 0/255, length=n-1)
  green1 <-seq(from = 0/255,to = 0/255, length=n-1) 
  blue1 <- seq(from = 0/255,to = 255/255, length=n-1)

  red2 <- seq(from = 0/255,to = 0/255, length=n-1)
  green2 <-seq(from = 0/255,to = 255/255, length=n-1) 
  blue2 <- seq(from = 255/255,to = 0/255, length=n-1)

  red <- c(red1, red2)
  green <- c(green1, green2)
  blue <- c(blue1, blue2)
  
  ## Returns the colormap
  rgb(red, green, blue)
	
}
