###########################################################################
## 
## %File: bg_correct.R
## %Plug-in: Normalization: Background correction
## %Version: R2:1.0
##
## %Authors: Adam Ameur, Stefan Enroth, Linnaeus Centre for Bioinformatics
## %Created: 27/4 2004 
## %Latest modified: 5/7 2006       
##
## %Description: This plug-in makes use of the local background intensities
##  of each spot to compensate for dye imbalances over the array. Background
##  correction can be done through estimating the background signal using simple
##  subtraction of the mean or median value of the local background (just outside
##  the spot) or using more complex measurements. The available methods are:
##  - Subtract: Background intensities are subtracted
##  - Half: Intensities that are less than 0.5 after background subtraction are
##    reset to be equal to 0.5.
##  - Minimum: Any intensity which is zero or negative after background subtraction
##    is set equal to half the minimum of the positive corrected intensities for
##    that array.
##  - Edwards: A log-linear interpolation method is used to adjust lower intensities
##    as in Edwards (2003)
##  - Kooperberg: A Bayesian model for background correction as in Kooperberg,
##    et al (2002).
##
## %Output: The plug-in creates a new data set with normalized spot intensities.
##
## %Parameters:
##  - Specify local background: Either median or mean
##  - Method: One of subtract, half, minimum, edwards or kooperberg
##  - Offset: A constant that may be added to the intensities before
##    log-transforming, so that the log-ratios are shrunk towards zero at the
##    lower intensities. This may eliminate or reverse the usual 'fanning' of
##    log-ratios at low intensities associated with local background subtraction.
##
###########################################################################

## Ignore warnings
options(warn = -1)

## Include functions for BASEfile read/write
source("../../../../configR2.R")
source(paste(config["plugins_R_dir"],"/common.R",sep=""))

## Load required libraries
library(limma)

## Load data and parameters
pars<-read.parameters()
data<-read.data()

print("Data loaded")

nrAssays<-length(data)

bg.data <- pars$bg.type
method  <- pars$method
offset <- as.numeric(pars$offset)

## Currently, 'movingmin' and 'normexp' are not working so they may
## not be selected in the plugin definition.
bg.correct.methods<-c("subtract", "half", "minimum", "edwards", "movingmin","normexp")

debug.output("Data and parameters loaded")

for (i in 1:nrAssays){
	
  ## Intensities are stored in R and G
  R<-data[[i]]$intensity1
  G<-data[[i]]$intensity2
	
  ## Backgrounds are stored in Rb and Gb
  if(bg.data == "median"){
    Rb<-data[[i]]$BCh1Median
    Gb<-data[[i]]$BCh2Median
  }

  if(bg.data == "mean"){
    Rb<-data[[i]]$BCh1Mean
    Gb<-data[[i]]$BCh2Mean
  }

  ## if the selected method is kooperberg
  if(method == "kooperberg"){

    ## Standard deviations for foregrounds
    SDR<-data[[i]]$FCh1SD
    SDG<-data[[i]]$FCh2SD
	
    ## Standard deviations for backgrounds
    SDRb<-data[[i]]$BCh1SD
    SDGb<-data[[i]]$BCh2SD
    
    ngenes<-length(data[[i]]$position)

    for(j in 1:ngenes){
      
      R[j]<-.expectedBayesianAdjustedFG(fg = R[j], bg = Rb[j], 
                                        sfg = SDR[j], sbg = SDRb[j])
      
      G[j]<-.expectedBayesianAdjustedFG(fg = G[j], bg = Gb[j], 
                                        sfg = SDG[j], sbg = SDGb[j])
    }

    ## Overwrite original intensities with new values
    data[[i]]$intensity1<-R
    data[[i]]$intensity2<-G
  }

  ## if the selected method is a mode in backgroundCorrect
  else if(method %in% bg.correct.methods){
		
    ## Create a RGList object	
    RG<-new("RGList",list(R=R,G=G,Rb=Rb,Gb=Gb))
		
    ## Run the background correction
    newRG<-backgroundCorrect(RG, method=method,offset = offset)

    ## Overwrite original intensities with new values
    data[[i]]$intensity1<-newRG$R
    data[[i]]$intensity2<-newRG$G
  }

  debug.output(paste(method,"method performed on array", i))
}

## Write output as transformation
write.output("stdout.txt",data)

debug.output("Finished")

clean.up()
