###################################################################################
##
## %File: bstat.R
## %Plug-in: Hypothesis Testing: B-statistics
## %Version: R2:1.0
##
## %Author: Adam Ameur, Linnaeus Centre for Bioinformatics
## %Latest modified: 5/7, 2006
## Built from code developed by Ola Spjuth, 2003.
##
## %Description: This plug-in calculates B-statistics (Smyth et al 2003) values for
##  ranking genes for each parameter in a 'design matrix'. A design matrix is a
##  representation of which differences are of interest for the particular study.
##  For information about how to create the design matrix, see (Glonek and Solomon 2004).
##  A high B-value indicates that the gene is likely to be differentially expressed.
##
## %Output: The plug-in returns a new data set, with extra columns named
##  'Bstat1','Bstat2',...,'BstatN<', were N is the number of parameters (columns in the
##  design matrix). In addition, a number of plots are produced for each parameter:
##  1. Volcano plot: M-values on the x-axis vs. B-values on the y-axis
##  2. MA-plot
##  3. Histogram over B-values
##  4. Distribution of B-values
##  5. Table containing the top differentially expressed genes
## 
## %Parameters: 
##  - Mark # top genes in plot: Number of top genes that are to be shown in plots 1 and 2. 
##  - Mark genes by: Either use Gene Symbol or Reporter ID to mark the genes. 
##  - Rows/Columns in design matrix: Dimensions of the design matrix. Rows should be the
##    same as number of arrays. Columns correspond to the number of parameters.
##  - Design matrix: Comma separated string with 0, 1 or -1. 
##  - Assumed proportion of DE genes: The default setting is 0.01, should not be tampered
##    with unless necessary. 
##  - Store B-values for: It is possible to store results only for genes with positive or
##    negative fold change. The default option is to store B-values for all genes.
##  - Minimum/Maximum stdev for DE genes: The default values are 0.1 and 4 respectively.
##    Should not be tampered with unless necessary.
##  - Only store genes with B-value above: Stores only results for genes with B-values
##    above a certain threshold. Useful for example when there are several different time
##    points or conditions.
##
## %Screen shots: bstat1.png (500), bstat2.png (400)
##
####################################################################################

## Ignore warnings
options(warn = -1)

## Include functions for BASEfile read/write
source("../../../../configR2.R")
source(paste(config["plugins_R_dir"],"/common.R",sep=""))
source(paste(config["plugins_R_dir"],"/rhtml.R",sep=""))

## Load required libraries
library(limma)

## Read data
pars<-read.parameters()
data<-read.data()

## Set up directory for the job where all plots are stored
picsDir<-"pics/"
dir.create(picsDir)

## #############################
##
## Store and check parameters
##
## #############################

MAX_PARAMETERS <- 6

genesInPlot<-as.numeric(pars$no.genesinplot)
markGenesBy<-as.character(pars$mark.genes.by)
design.r<-as.numeric(pars$ndesign.r.in)
design.c<-as.numeric(pars$ndesign.c.in)
proportion<-as.numeric(pars$proportion)
fold.change.in<-as.character(pars$fold.change.in)
stdev.min<-as.numeric(pars$stdev.min)
stdev.max<-as.numeric(pars$stdev.max)
p.adj<-as.character(pars$p.adj)
bstat.cut.off<-NULL
nr.top.cut.off<-NULL

if(pars$bstat.cut.off != ""){
  if(is.na(as.numeric(pars$bstat.cut.off))){
    die("bstat.R", "Parameter bstat cut-off must be numeric.")
  }
  bstat.cut.off<-as.numeric(pars$bstat.cut.off)
}

if(pars$nr.top.cut.off != ""){
  if(is.na(as.numeric(pars$nr.top.cut.off))){
    die("bstat.R", "Parameter nr top cut-off must be numeric.")
  }
  nr.top.cut.off<-as.numeric(pars$nr.top.cut.off)
}
  
fold.change.min<-NULL
fold.change.max<-NULL

if(fold.change.in == "pos"){
  fold.change.min<-0
}

if(fold.change.in == "neg"){
  fold.change.max<-0
}

if(!((proportion >= 0) && (proportion <= 1))){
  die("bstat.R", "Propotion of DE genes must be a number between 0 and 1.")
}

if(!(stdev.min > 0) || !(stdev.max > 0) || !(stdev.min < stdev.max)){
  die("bstat.R", "Wrong limits on standard deviation for DE genes.")
}

stdev.coef.lim<-c(stdev.min, stdev.max)

if(!(p.adj %in% c("fdr","none","holm"))){
  die("bstat.R", "P-value adjustment method not specified.")
}

debug.output("Data and parameters loaded")

## ###############################
##
## Create design matrix from input
##
## ###############################

if(design.r  == 0){
  die("bstat.R", "Please specify number of rows in design matrix.")
}

if(design.c == 0){
  die("bstat.R", "Please specify number of columns in design matrix.")
}
if(design.c > MAX_PARAMETERS && !is.null(bstat.cut.off)){
  die("bstat.R", "Too many columns in design matrix.")
}

tmp<-as.numeric(unlist(strsplit(pars$design.in,",")))

if(design.c*design.r != length(tmp)){
  die("bstat.R", "Design matrix doesn't agree with row and column dimensions.")
}

design<-matrix(nrow=design.r, ncol=design.c, data=tmp, byrow=TRUE)

debug.output("Design matrix created")

## Create an RG object from the loaded data.
## Requires that all arrays have the same layout.
RG<-make.RG.matrix(data, extended = TRUE)

debug.output("RG object created")

## Convert to MA
MA <- MA.RG(RG)			

no.assays<-length(data)			

## ####################
##
## Run the B-statistics
##
## ####################

## Fit a linear model to the design matrix
fitLM<-lmFit(MA,design)

debug.output("Linear model created")

## Run the empirical Bayes method
fit<-eBayes(fitLM, proportion = proportion, stdev.coef.lim=stdev.coef.lim)

debug.output("eBayes function executed")

## #############################################
##
## Create result plots for each B-stat parameter
##
## #############################################

options(digits=3)
nr.genes<-dim(MA$M)[1]
nr.params<-length(design[1,])
nr.arrays<-length(data)

outfileNames<-paste(picsDir,"bstat",1:nr.params,".png",sep="")
heightInches<-13
widthInches<-13

Bstat.names<-paste("Bstat",1:nr.params,sep="")
Bstat.values<-matrix(NA, nrow=nr.genes, ncol=nr.params, dimnames=list(RG$position,Bstat.names))

for (i in 1:nr.params){

  coef<-array()
  lods<-array()
	
  if(nr.params == 1){
    lods<-fit$lods
    coef<-fit$coef
  }

  else{
    lods<-fit$lods[,i]
    coef<-fit$coef[,i]
  }

  Bstat.values[,i]<-round(lods,digits=2)

  ## Only store results for genes with fold change in the given range
  if(is.numeric(fold.change.min)){
    Bstat.values[,i][which(coef<fold.change.min)]<-NA
  }
  if(is.numeric(fold.change.max)){
    Bstat.values[,i][which(coef>fold.change.max)]<-NA
  }

  if(!is.null(nr.top.cut.off)){
    Bstat.values[,i][order(Bstat.values[,i], decreasing=TRUE)][-(1:round(nr.top.cut.off))] <- NA
  }
  
  ## Order genes on B-stat score
  ord <- order(lods,decreasing=TRUE)

  ## Get the top N scoring genes 
  top <- ord[1:genesInPlot]

  Bstat.title<-NULL
  gene.labels<-NULL
  
  if(markGenesBy == "repId"){
    Bstat.title<-"B-stat plot (with reporter ids as lables)"
    gene.labels<-as.character(RG$repId[top])
  }
  
  if(markGenesBy == "geneSymbol"){
    Bstat.title<-"B-stat plot (with gene symbols as lables)"
    gene.labels<-as.character(RG$geneSymbol[top])
  }

  if(is.null(gene.labels)){
    die("bstat.R", "Gene labels must be either Reporter ID or Gene Symbol!")
  }
  
  ## Set layout for the result plots
  bitmap(file=outfileNames[i], type = "png16m", height = heightInches, width = widthInches, res = 72)
  layout(matrix(c(1,2,2,1,3,4), nrow=3, ncol=2), widths=c(10,6), heights=c(10,5,5))

  ## B-stat plot with reporter ids or gene symbols for top genes

  par(mar=c(3,2,2,1))

  plot(coef,lods, pch=16, cex=0.1, main=Bstat.title, xlab="M", ylab="B-statistics")
  text(coef[top],lods[top],labels=gene.labels, col="blue", cex=0.8)

  ## MA-plot with reporter ids or gene symbols for top genes
  M<-coef
  A<-fit$Amean

  par(mar=c(3,2,2,1))

  plot(A,M,main="MA-plot", pch=16, cex=0.1)
  text(A[top],M[top], labels=gene.labels, col="blue", cex=0.8)

  ## Histogram of B-statistics values
  hist(lods, main="Histogram of B-statistics", density=10, angle=45,counts=TRUE, ylab="Number of reporters", xlab="B-statistics value")

  par(mar=c(3,2,2,1))
  
  plot(lods[ord], main="Number of genes over cut-off", xlab="Number of genes", ylab="B-statistics cut-off")
  
  ycoord<-round(lods[top][genesInPlot], digits=2)
  abline(h=ycoord, col="blue")
  text(nr.genes/2,y=ycoord, pos=1, paste("cut-off at ", as.character(ycoord), " for top ", genesInPlot, " genes", sep=""), cex=0.8)
  
  dev.off()

  debug.output(paste("Created results for parameter",i))
}

## #######################################################
##
## Print resulting data set to the file "stdout.txt".
## The results are printed in different ways depending on
## whether the bstat cut-off is set or not.
##
## i) B cut-off set: All spots are filtered so that only
##    the ones above the cut-off remains. No extracolumns.
##
## ii) B cut-off unset: All spots in the data are stored,
##     with B-statistics values are in extra columns. 
##
## #######################################################

for(i in 1:nr.params){
  XcolName<-Bstat.names[i]
  tmp.cmd<-paste("RG$",XcolName,"<-Bstat.values[,",i,"]",sep="")
  eval(parse(text=tmp.cmd))
}

if(is.null(bstat.cut.off) && is.null(nr.top.cut.off)){
  write.output.from.RG("stdout.txt",RG,extraColumns=Bstat.names)
  debug.output("Results created, all B-values stored")
}

if(!is.null(bstat.cut.off) || !is.null(nr.top.cut.off)){
  if(is.null(bstat.cut.off)){
    bstat.cut.off <- -1000
  }
  write.output.from.RG.bCutOff("stdout.txt",RG,extraColumns=Bstat.names, design = design, bstat.cut.off = bstat.cut.off)
  debug.output("Results created, B-values above cut-off stored")
}
  
## ###########################
##
## Create HTML result page 
##
## ###########################

resFile<-"Results.html"

## Create files with tables for the top N genes

tableFiles<-paste(picsDir,"topTable",1:nr.params,".html", sep="")
dataFiles<-paste(picsDir,"rankedGenes",1:nr.params,".txt", sep="")

geneInfo<-data.frame(cbind(RG$reporter,RG$repId,RG$geneSymbol))
colnames(geneInfo)<-c("reporter","reporterId","geneSymbol")

for (i in 1:nr.params){

  ## Create .txt file with all B-stat data
  Bres <- toptable(coef=i, number=nr.genes, geneInfo, fit=fitLM, eb=fit, adjust=p.adj)
  Bres[,4:8] <- signif(Bres[4:8], digits=3)
  write.table(Bres, eol="\n",sep="\t", file=dataFiles[i], row.names=FALSE, na="", quote=FALSE)
      
  ## Create files with tables for the top N genes
  cat("<html>\n", file=tableFiles[i])
  cat(styleStr, file=tableFiles[i], append=TRUE)
  cat(paste("<h1 align=center>Top ",genesInPlot," genes from B-statistics for parameter ",i,"</h1>",sep=""), file=tableFiles[i], append=TRUE)
  cat("<table width=600 border=\"1\" frame=\"border\" rules=\"none\"><tr bgcolor=\"ddddff\"><td>", file=tableFiles[i], append=TRUE)
  BresSub <- toptable(coef=i, number=genesInPlot, geneInfo, fit=fitLM, eb = fit, adjust=p.adj)
  BresSub[,4:8] <- signif(BresSub[4:8], digits=3)
  write.table(BresSub, eol="</td><tr bgcolor=\"f0f0ff\">\n<td>",sep="</td><td>", file=tableFiles[i], row.names=FALSE, na="", quote=FALSE, append=TRUE)
  cat("</td></tr></table>", file=tableFiles[i], append=TRUE)
  
  cat(paste("<p><a href = \"rankedGenes",i,".txt\">All ranked genes in text file</a>", sep=""), file=tableFiles[i], append=TRUE)
    
  cat("</body></html>", file=tableFiles[i], append=TRUE)


}


blankname<-paste(picsDir,"blank.png", sep="")

get.blank(fname = blankname, width = widthInches, height = heightInches)

heads <- data.frame("Parameter","Genes")
tableData<-paste("<a href=\"",tableFiles,"\">Top",genesInPlot,"</a>", sep="")
data<-data.frame(tableData)

hw=widthInches*72
hh=heightInches*72

resultTable<-pic.table(header=heads, data = data, pics = outfileNames,width = hw,height = hh, use.names=FALSE, blankname = blankname)

cat("<html>\n", file=resFile)
cat(styleStr, file=resFile, append=TRUE)

cat("<h1 align=center>Results from B-statistics</h1>", file=resFile, append=TRUE)

cat(resultTable, file=resFile, append = TRUE)

cat("</body></html>", file=resFile, append=TRUE)

debug.output("Finished")

clean.up()



