################################################################################
##
## File: common.R
## Version: R2:1.0
##
## Creator: Adam Ameur, Linnaeus Centre for Bioinformatics, 10/3 2004
## Latest modified: 6/7 2006
##
## Description: This file contains functions that are needed for all 
## R programs that are started by the perl script R2.pl. 
##
## The functions read.parameters, read.assays and read.data is for 
## loading the files created from R.pl into R data structures. 
## A description of the data structures follows below:
##
## 	Parameters - Created from the file "parameters.txt"
##
## The function call params<-read.parameters() stores all parameters 
## to in the structure params. The parameter is accessed by its name, so
## that params$name is the value of the parameter. Parameter values are 
## stored as strings. It is important to note that debug is initiated
## in the read.parameters() function according to the value of the hidden
## string argument debug.level.
##
## 	Assays - Created from the file "assays.txt"
##
## The function call assays<-read.assays() stores all information about 
## the assays in the structure assays. The assays structure is a matrix
## with the column fields from the assays section in the BASEfile as 
## columns, and the different assays as rows.
##
## 	Data - Created from all files dataXXX.txt
##
## The function call data<-read.data() stores all data for the different 
## assays in the structure data. The structure data is a list with one 
## element for every assay. Data for array number i is stored in data[[i]], 
## and data for a specific field for that assay is stored in data[[i]]$field. 
## The field can for example be intensity1, intensity2, BCh1Mean or whatever
## was exported to the BASEfile. The assay id is stored in data[[i]]$assayId
##
##      Layout - Created from the file layout.txt
##
## The function call layout<-read.layout() stores the grid layout for each
## assay in the structure layout.
##
##      Experiment - Created from the file experiment.txt
##
## The function call experiment<-read.experiment() stores the experiment name
## and in the future also the experiment design in the structure experiment.
##
## There are also functions for creating a BASEfiles, either from the structure
## data described above or RG matrices. 
##
########################################################################

source(paste(config["plugins_R_dir"],"/debug.R",sep=""))

## Function for creating a structure parameters from 'parameters.txt'
## Checks if the debug parameter is set and initializes profiling and/or
## debug in such case.
read.parameters<-function(){
  
  parameterFile<-"parameters.txt"
  parameterData<-read.table(parameterFile, header=TRUE, sep="\t", comment.char="")
  
  print("Parameters loaded")
  print(parameterData)

  pars<-list()

  for (i in 1:length(parameterData[,1])){
    parName<-parameterData[i,1]
    parValue<-parameterData[i,2]

    ## The value for the parameter 'fname' is the name of an 
    ## R function and may not be stored in the data strucure
    if(parName != "fname"){
      tmp.cmd<-paste("pars$",parName,"<-\"",parValue,"\"",sep="")
      eval(parse(text=tmp.cmd))
    }
  }

  ## Remove the file 'parameters.txt'
  unlink(parameterFile, recursive=FALSE)

  ## Initialize debug/profiling
  if(!is.null(pars$debug)){
    ## only debug 
    if(as.integer(pars$debug) == 1){
      profiling.on()
    }
    
    ## profiling
    if(as.integer(pars$debug) == 2){
      debug.init()
    }
    
    ## debug and profiling 
    if(as.integer(pars$debug) == 3){
      profiling.on()
      debug.init()
    }

    ## Initialize debug level
    if (!is.null(pars$level.debug)) {
      debug.level.init(as.numeric(pars$level.debug))
    }
  }

  ## Store process id in a temporary file
  debug.store.pid()
  
  return(pars)
}

## Function for creating a structure assays from 'assays.txt'
read.assays<-function(){
  
  assaysFile<-"assays.txt"
  assaysData<-read.table(assaysFile, header=TRUE, sep="\t", comment.char="")

  print("Assays information loaded")
  print(assaysData)

  ## Remove the file 'assays.txt'
  unlink(assaysFile, recursive=FALSE)

  return(assaysData)

}


## Function for creating a structure layout from 'layout.txt'
read.layout<-function(){

  layoutFile<-"layout.txt"
  layoutData<-read.table(layoutFile, header=TRUE, sep="\t", comment.char="")

  print("Layout information loaded")
  print(layoutData)

  layoutArrayIds<-as.vector(layoutData["id"][[1]])
  layoutArrayNames<-dimnames(layoutData)[[2]]

  layouts<-list()
  
  for(i in 1:length(layoutArrayIds)){
    layout<-list();
    for(j in 1:length(layoutArrayNames)){
      name<-layoutArrayNames[j]
      tmp.cmd<-paste("layout$",name,"<-layoutData$",name,"[",i,"]", sep="")
      eval(parse(text=tmp.cmd))
    }

    layouts[[i]]<-as.vector(layout)
}

  ## Remove the file 'layout.txt'
  unlink(layoutFile, recursive=FALSE)

  return(layouts)

}

## Function for loading experiment design from 'experiment.txt'
read.experiment<-function(){

  experimentFile<-"experiment.txt"
  experimentData<-read.table(experimentFile, header=TRUE, sep="\t", comment.char="")

  experimentNames<-dimnames(experimentData)[[2]]

  print("Experiment information loaded")
  print(experimentData)

  experiment<-list()

  for(i in 1:length(experimentNames)){
    name<-experimentNames[i]
    tmp.cmd<-paste("experiment$",name,"<-as.character(experimentData$",name,")",sep="")
    print(tmp.cmd)
    eval(parse(text=tmp.cmd))
  }

  ## Remove the file 'experiment.txt'
  unlink(experimentFile, recursive=FALSE)

  return(experiment)

}

## Function for creating a structure data from the files 'dataXXX.txt',
## where XXX denotes the different assay ids.
read.data<-function(){

  ## Compute the total number of assays
  assaysData<-read.assays()

  nrassays<-dim(assaysData)[1]

  assays<-1:nrassays

  assayIds<-as.vector(assaysData["id"][[1]])
  assayNames<-as.vector(assaysData["name"][[1]])

  allassays<-list()
  i<-1

  ## Load all 'dataXXX.txt' files
  for (assayNumber in assays) {

    assayId<-assayIds[assayNumber]
    assayName<-assayNames[assayNumber]

    dataFileName<-paste("data",assayId,".txt",sep="")
	
    tempData<-read.table(dataFileName, header=TRUE, sep="\t", comment.char="",quote = "")

    names<-dimnames(tempData)[[2]]

    ## Store information about the current assay.
    assay <- list()

    for (j in 1:length(names)){
      tmp.cmd<-paste("assay$",names[j],"<-tempData$",names[j], sep="")
      eval(parse(text=tmp.cmd))
    }

    assay$assayNumber<-assayNumber
    assay$assayId<-assayId
    assay$assayName<-assayName
    
    allassays[[i]]<-assay

    i<-i+1

    ## Remove current 'dataXXX.txt' file
    unlink(dataFileName, recursive=FALSE)
  }

  return(allassays)

}

## This function takes a structure data as input and prints out a
## serial BASEfile. In most cases the function is used for printing 
## output from a plugin to stdout.txt. If extra columns are stored
## in the data, they may be printed to the outfile by specifing 
## the extra column headers in the extra columns list. Note that 
## this function only works for 2 channel data!
write.output<-function(fname, assays, extraColumns = c()){

  nrAssays <- length(assays)
  
  ## ######################
  ##
  ## Print BASEfile header
  ##
  ## ######################

  write("BASEfile", file=fname, append=FALSE)

  ## #####################
  ##
  ## Print spot sections
  ##
  ## #####################

  ## Add names of extra columns if there are any

  nrXCol<-length(extraColumns)

  extraColumnHeader<-""

  if(nrXCol > 0){
    for(e in 1:nrXCol){
      extraColumnHeader<-paste(extraColumnHeader,"\t",extraColumns[e], sep="")
    }
  }

  for (index in 1:nrAssays){

    write("section\tspots", file=fname, append=TRUE)
    write("columns\tposition\treporter\tassayData", file=fname, append=TRUE)
    write("channels\t2", file=fname, append=TRUE)

    ## header names	
    write(paste("assayFields\tintensity1\tintensity2",extraColumnHeader,sep=""), file=fname, append=TRUE)

    ## the extra headers for exporting additional data	
    if(nrXCol > 0){
      cat("setExtraFloats",file=fname,append = TRUE)
      cat(extraColumnHeader, file = fname,append = TRUE,sep ="",fill = TRUE)
    }        

    assayInfo<-paste("assays\t",assays[[index]]$assayId)
    write(assayInfo, file=fname, append=TRUE)

    write("%", file=fname, append = TRUE)

    ## expand the matrix with the extra data columns
    nrRows<-length(assays[[index]]$position)
    nrColumns<-4+nrXCol
	
    out<-matrix(data=NA,nrow=nrRows,ncol=nrColumns)
	
    out[,1]<-as.character(as.integer(assays[[index]]$position))
    out[,2]<-as.character(as.integer(assays[[index]]$reporter))
    out[,3]<-as.character(assays[[index]]$intensity1)
    out[,4]<-as.character(assays[[index]]$intensity2)

    ## Print the extra data to out file
    if(nrXCol > 0){
      for(e in 1:nrXCol){			
        tmp.cmd<-paste("tmpXCol<-assays[[",index,"]]$",extraColumns[e],sep="")
        print(tmp.cmd)
        eval(parse(text=tmp.cmd))
        if(length(tmpXCol) != nrRows){
          die("common.R", "Unable to write extraColumns to stdout.txt")
        }
        out[,4+e]<-as.character(tmpXCol)
      }
    }
        
    write.table(out,file=fname,quote=FALSE,col.names=FALSE,row.names=FALSE, na="", append = TRUE,sep="\t")
    write("", file=fname, append = TRUE)

  }

  print("BASEfile printed to stdout.txt")
}

## This function takes a RG matrix as input and prints out a
## serial BASEfile. In most cases the function is used for printing 
## output from a plugin to stdout.txt. If extra columns are stored
## in the data, they may be printed to the outfile by specifing 
## the extra column headers in the extra columns list. Note that 
## this function only works for 2 channel data!
write.output.from.RG<-function(fname, RG, extraColumns = c(), write.assays.section = FALSE){

  nrAssays <- length(RG$assayIds)

  ## ######################
  ##
  ## Print BASEfile header
  ##
  ## ######################
  
  write("BASEfile", file=fname, append=FALSE)

  ## ################################
  ##
  ## Print assays section (optional)
  ##
  ## ################################
  
  if(write.assays.section){
				
    write("section\tassays", file=fname, append=TRUE)
    countLine<-paste("count\t",nrAssays,sep="")
    write(countLine, file=fname, append=TRUE)
    write("columns\tid\tname", file=fname, append=TRUE)
    write("annotationColumns", file=fname, append=TRUE)
    
    write("%", file=fname, append=TRUE)
	
    for(i in 1:nrAssays){
      assayStr<-RG$assayIds[[i]] # Assay id
      assayStr<-paste(assayStr,RG$assayNames[[i]],sep="\t")
      
      write(assayStr,file=fname, append=TRUE)
    }
    write("",file=fname, append=TRUE)
  }
  
  ## #####################
  ## 
  ## Print spot sections
  ##
  ## #####################

  nrXCol<-length(extraColumns)
  
  extraColumnHeader<-""
  
  if(nrXCol > 0){
    for(e in 1:nrXCol){
      extraColumnHeader<-paste(extraColumnHeader,"\t",extraColumns[e], sep="")
    }
  }

  print(paste("Extra column header: ",extraColumnHeader,sep=""))
  
  for (index in 1:nrAssays){
    
    write("section\tspots", file=fname, append=TRUE)
    write("columns\tposition\treporter\tassayData", file=fname, append=TRUE)
    write("channels\t2", file=fname, append=TRUE)

    ## header names	

    write(paste("assayFields\tintensity1\tintensity2",extraColumnHeader,sep=""), file=fname, append=TRUE)

    ## the extra headers for exporting additional data
    if(nrXCol > 0){
      cat("setExtraFloats",file=fname,append = TRUE)
      cat(extraColumnHeader, file = fname,append = TRUE,sep ="",fill = TRUE)
    }        
	
    assayInfo<-paste("assays\t",RG$assayIds[[index]])
    write(assayInfo, file=fname, append=TRUE)
    
    write("%", file=fname, append = TRUE)

    nrRows<-length(RG$position)
    nrColumns<-4+nrXCol
    
    out<-matrix(NA,nrow=nrRows,ncol=nrColumns)
    
    out[,1]<-as.character(as.integer(RG$position))
    out[,2]<-as.character(as.integer(RG$reporter))
    out[,3]<-as.character(RG$R[,index])
    out[,4]<-as.character(RG$G[,index])
    
    if(nrXCol > 0){
      
      for(e in 1:nrXCol){			
        tmp.cmd<-paste("tmpXCol<-RG$",extraColumns[e],sep="")

        eval(parse(text=tmp.cmd))

        if(length(tmpXCol) != nrRows){
          die("common.R", "Unable to write extraColumns to stdout.txt")
        }

        out[,4+e]<-as.character(tmpXCol)
       
      }

    }

    write.table(out,file=fname,quote=FALSE,col.names=FALSE,row.names=FALSE, na="", append = TRUE,sep="\t")

    write("", file=fname, append = TRUE)

  }

  print("BASEfile printed to stdout.txt")
}


## This function takes a RG matrix as input and prints out a
## serial BASEfile for results created by bstat, when the bstat
## cut-off has been set. The corresponding design matrix must be
## given, and only the spots with values above the cut-off are
## printed. The extra columns are not printed, only filtered on!!!
write.output.from.RG.bCutOff<-function(fname, RG, extraColumns = c(), write.assays.section = FALSE, design, bstat.cut.off){

  nrAssays <- length(RG$assayIds)

  ## ######################
  ##
  ## Print BASEfile header
  ##
  ## ######################
  
  write("BASEfile", file=fname, append=FALSE)

  ## ################################
  ##
  ## Print assays section (optional)
  ##
  ## ################################
  
  if(write.assays.section){
				
    write("section\tassays", file=fname, append=TRUE)
    countLine<-paste("count\t",nrAssays,sep="")
    write(countLine, file=fname, append=TRUE)
    write("columns\tid\tname", file=fname, append=TRUE)
    write("annotationColumns", file=fname, append=TRUE)
    
    write("%", file=fname, append=TRUE)
	
    for(i in 1:nrAssays){
      assayStr<-RG$assayIds[[i]] # Assay id
      assayStr<-paste(assayStr,RG$assayNames[[i]],sep="\t")
      
      write(assayStr,file=fname, append=TRUE)
    }
    write("",file=fname, append=TRUE)
  }
  
  ## #####################
  ## 
  ## Print spot sections
  ##
  ## #####################

  nrXCol<-length(extraColumns)
    
  for (index in 1:nrAssays){
    
    write("section\tspots", file=fname, append=TRUE)
    write("columns\tposition\treporter\tassayData", file=fname, append=TRUE)
    write("channels\t2", file=fname, append=TRUE)

    ## header names	

    write(paste("assayFields\tintensity1\tintensity2",sep=""), file=fname, append=TRUE)

    assayInfo<-paste("assays\t",RG$assayIds[[index]])
    write(assayInfo, file=fname, append=TRUE)
    
    write("%", file=fname, append = TRUE)

    nrRows<-length(RG$position)
    nrColumns<-4
    
    out<-matrix(NA,nrow=nrRows,ncol=nrColumns)
    
    out[,1]<-as.character(as.integer(RG$position))
    out[,2]<-as.character(as.integer(RG$reporter))
    out[,3]<-as.character(RG$R[,index])
    out[,4]<-as.character(RG$G[,index])
    
    if(nrXCol > 0){
      significants <- NULL
      for(e in 1:nrXCol){			
        tmp.cmd<-paste("tmpXCol<-RG$",extraColumns[e],sep="")

        ## For Bstat, no score should be printed for the
        ## arrays that don't measure the parameter.
        if(!is.null(bstat.cut.off)){
          if(design[index,e] == 0){
            tmp.cmd<-paste("tmpXCol<-rep(\"\", length(RG$",extraColumns[e],"))",sep="")
          }
        }

        eval(parse(text=tmp.cmd))

        if(length(tmpXCol) != nrRows){
          die("common.R", "Unable to write extraColumns to stdout.txt")
        }

        if(!is.null(bstat.cut.off)){
          significants <- union(which(tmpXCol >= bstat.cut.off), significants)
          tmpXCol[which(tmpXCol < bstat.cut.off)] <- ""
        }
      }

      if(!is.null(bstat.cut.off)){
        out[-significants, ] <- ""
        out <- unique(out)
      }
    }

    write.table(out,file=fname,quote=FALSE,col.names=FALSE,row.names=FALSE, na="", append = TRUE,sep="\t")

    write("", file=fname, append = TRUE)

  }

  print("BASEfile printed to stdout.txt")
}


## Function for writing error message to stderr and aborting the R-program.
## Takes the function in which the error occured and an error message as input.
die<-function(fName, errMsg){
  cat(paste("Error in ",fName,"\n",sep=""),file=stderr())
  cat(paste(errMsg,"\n",sep=""),file=stderr())
  flush(stderr())
  q(status=-1)	
}

## Function for writing a warning message to stderr.
## Takes the function in which the warning occured and an error message as input.
warning<-function(fName, errMsg){
  cat(paste("Warning in ",fName,"\n",sep=""),file=stderr())
  cat(paste(errMsg,"\n",sep=""),file=stderr())
  flush(stderr())
}

## This function takes serial data as input and converts it to an RG matrix.
## Requires that all arrays have the same layout, i.e that that every position
## measures the same reporter on all arrays. Position and reporter information
## is always stored in the reulting RG structure and if the option extended is
## set to TRUE, reporter IDs and gene symbols are also stored.
make.RG.matrix <- function(data, extended = FALSE){

  posToGeneInfo<-new.env(hash=TRUE)
  
  nrAssays<-length(data)
  
  for (i in 1:nrAssays){

    assayPosition<-as.character(data[[i]]$position)
    assayReporter<-as.character(data[[i]]$reporter)
    repId<-NULL
    geneSymbol<-NULL
    
    if(extended == TRUE){
      repId<-as.character(data[[i]]$reporterId)
      repId[which(repId == "")]<-NA
      geneSymbol<-as.character(data[[i]]$geneSymbol)
      geneSymbol[which(geneSymbol == "")]<-NA
    }

    for(index in 1:length(assayPosition)){

      pos<-assayPosition[index]
      rep<-assayReporter[index]
      
      if(exists(pos, posToGeneInfo)){
        old.rep<-get(pos,posToGeneInfo)
        if(!(old.rep[1] == rep)){
          die("make.RG.matrix",paste("Position ", pos, "has different reporters on the arrays.", sep=""))
        }
      }
      
      ## Add information about a new position to the environment
      else{
        value<-c(rep) 
        if(extended == TRUE){
          value<-c(rep, repId[index], geneSymbol[index])
        }
        assign(pos, value, posToGeneInfo)
      }
    }
  }

  allPositions<-ls(posToGeneInfo)
  nrPositions<-length(allPositions)
  
  allReporters<-array()
  allReporterIds<-array()
  allGeneSymbols<-array()

  geneInfo <- mget(allPositions, posToGeneInfo) 
  
  allReporters <- sapply(1:length(geneInfo), function(i){geneInfo[[i]][1]})
  
  if(extended == TRUE){
    allReporterIds <- sapply(1:length(geneInfo), function(i){geneInfo[[i]][2]})
    allGeneSymbols <- sapply(1:length(geneInfo), function(i){geneInfo[[i]][3]})
  }
    
  assayIds<-list()
  assayNames<-list()
  
  intensity1Matrix<-matrix(data=NA, nrow=nrPositions, ncol=nrAssays)
  intensity2Matrix<-matrix(data=NA, nrow=nrPositions, ncol=nrAssays)
  
  for(i in 1:nrAssays){
    
    assayPosition<-data[[i]]$position
    
    matchingPositions<-match(assayPosition,allPositions)
    
    assayInt1<-data[[i]]$intensity1
    assayInt2<-data[[i]]$intensity2
    
    newInt1<-array(data=NA, dim=length(allPositions))
    newInt2<-array(data=NA, dim=length(allPositions))
    
    newInt1[matchingPositions]<-assayInt1
    newInt2[matchingPositions]<-assayInt2
    
    intensity1Matrix[,i]<-newInt1
    intensity2Matrix[,i]<-newInt2
    
    assayIds[[i]]<-data[[i]]$assayId
    assayNames[[i]]<-data[[i]]$assayName
  }

  RG<-list(position=allPositions,reporter=allReporters,assayIds=assayIds,assayNames=assayNames,R=intensity1Matrix,G=intensity2Matrix)
	
  if(extended == TRUE){
    RG$repId<-allReporterIds
    RG$geneSymbol<-allGeneSymbols
  }

  return(RG)
  
}

## This function removes all files created in the parsing except
## dataXXX.txt and assays.txt. 
clean.up<-function(){
  unlink("parameters.txt", recursive=FALSE)
  unlink("layout.txt",recursive=FALSE)
  unlink("experiment.txt",recursive=FALSE)
  unlink("R-pid.txt",recursive=FALSE)
  if(!(DEBUG)){
    unlink("R-debug.txt",recursive=FALSE)
  }
  if(PROFILING){
    write.profiling()
  }
}








