#############################################################################
##
## %File: correlations.R
## %Plug-in: Visualization: Array similarity plots
## %Version: R2:1.0
##
## %Author: Adam Ameur, Linnaeus Centre for Bioinformatics
## %Latest modified: 5/7 2006
##
## %Description: The plug-in displays the similarity between all arrays in an
##  experiment. Arrays that theoretically should show high similarity (for
##  example replicates) should ideally group together in these plot. However,
##  this is not always the case, especially before data has been normalized.
##  The tool can therefore be used for quality assessment of the data. If the
##  array similarities are not as expected, it might be necessary to perform
##  additional normalization in order to reduce systematic variation in the data.
##  - Principal Components Analysis (PCA): PCA is a linear transformation that
##    transforms the data to a new coordinate system such that the greatest
##    variance by any projection of the data comes to lie on the first coordinate
##    (called the first principal component), the second greatest variance on the
##    second coordinate, and so on. PCA can be used for dimensionality reduction
##    in a dataset while retaining those characteristics of the dataset that
##    contribute most to its variance, by keeping lower-order principal components
##    and ignoring higher-order ones.
##  - Hierarchical clustering: An unsupervised method for the classification of
##    similar arrays into different groups. The results are represented by a tree
##    structure (or dendrogram), with individual elements at one end and a single
##    cluster with every element at the other.
##  - Pairwise correlations: Strength and direction of the linear relationship
##    between each possible pair of arrays.
##
## %Output: The plug-in produces three types of plots:
##  - PCA plot: Projection of arrays onto the plane spanned by the two first
##    principal components. Arrays that are close in this 2D plot are likely
##    to contain similar data.
##  - Clustering plot: Dendrogram and heat map containing the results of
##    hierarchical clustering.
##  - Correlation plots: One plot is produced for each pair of arrays, with
##    the log2-ratios plotted against each other. The correlation coefficient
##    is indicated by a red line in each plot. The correlations are summarized
##    in a table.
##
## %Parameters:
##  - Distance metric in clustering: A number of options are available.
##  - Clustering method: A number of options are available.
##  - Correlation method: The options are pearson, spearman and kendall.
##  - Reorder correlation table?: If set to yes, the rows and columns of
##    the correlation table will be ordered according to the results from
##    hierarchical clustering. Otherwise the table will ordered by array names.
##
## %Screen shots: arraysim1.png (500), arraysim2.png (500)
##
#############################################################################

## ################
##
## Main program
##
## ################

## Ignore warnings
options(warn = -1)

source("../../../../configR2.R")
source(paste(config["plugins_R_dir"],"/common.R",sep=""))
source(paste(config["plugins_R_dir"],"/correlations.inc.R",sep=""))

## Load data
pars<-read.parameters()
data<-read.data()

compMatrix <- matrix(1,nrow=length(data),ncol=length(data))

debug.output("Data loaded")

arrayCorrelationPlots(data = data, pars = pars, compMatrix = compMatrix)

debug.output("Finished")

clean.up()

