######################################################################
##
## File: correlations.inc.R
## 
## Version: R2: 1.0
##
## Author: Adam Ameur, 15/5 2006
## Last modified: 5/7 2006
##
## Linnaeus Centre for Bioinformatics
##
## Descriprion: This file contains help functions for visualization of
## between array correlation. 
##
## 1) PCA - Principal Component Analysis
## 2) Hierarchical clustering of samples
## 3) Pairwise correlation plots
##
#######################################################################


## Performs PCA, Hierarchical clustering and pairwise correlation and
## prints results to a HTML page. The arguments data and pars must have
## the same structure as produced by the parsing function in common.R.
## The compMatix is a binary n x n matrix that specifies the arrays to
## be pairwise correlated. If the continueHTML argument is set, the
## HTML-page is not terminated allowing additional results to be printed
## to the same page.
arrayCorrelationPlots <- function(data, pars, compMatrix, continueHTML = FALSE){

  source(paste(config["plugins_R_dir"],"/rhtml.R",sep=""))
  
  picsDir<-"pics/"
  dir.create(picsDir)

  library(limma)
  library(sma)
  
  ## ##########################
  ##
  ## Read and check parameters
  ##
  ## ##########################
  
  plotPCAandClust <- as.character(pars$plotPCAandClust)
  clustDistMethod <- as.character(pars$distMethod)
  clustMethod <- as.character(pars$clustMethod)
  corMethod <- pars$corMethod
  reorderIn <- as.character(pars$reorder)
  
  PCAandClust <- TRUE
  
  if(!(plotPCAandClust %in% c("yes","no"))){
    die("correlations.inc.R", "Hidden parameter must be 'yes' or 'no'")
  }

  if(plotPCAandClust == "no"){
    PCAandClust <- FALSE
  }

  reorder <- TRUE

  if(!(reorderIn %in% c("yes","no"))){
    die("correlations.inc.R", "Reorder parameter must be 'yes' or 'no'")
  }

  if(reorderIn == "no"){
    reorder <- FALSE
  }
    
  distMethods <- c("euclidean", "maximum", "manhattan","canberra", "binary")
  
  if(!(clustDistMethod %in% distMethods)){
    die("correlations.inc.R", paste("Distance method",clustDistMethod,"not supported."))
  }

  corMethods <- c("pearson", "kendall", "spearman")

  if(!(corMethod %in% corMethods)){
    die("correlations.inc.R", paste("Correlation method",corMethod,"not supported."))
  }

  clustMethods <- c("ward","single", "complete", "average", "mcquitty","median","centroid")

  if(!(clustMethod %in% clustMethods)){
    die("correlations.inc.R", paste("Clustering method",corMethod,"not supported."))
  }
  
  nrAssays <- length(data)

  assayNames <- 1:nrAssays
  realAssayNames <- sapply(1:nrAssays, function(i){data[[i]]$assayName})

  debug.output("Parameters loaded and checked")
  
  ## Create an RG object from the loaded data.
  RG<-make.RG.matrix(data, extended=TRUE)
  
  debug.output("RG object created")

  M <- as.matrix(log2(RG$R/RG$G))

  rowIndex <- 1:nrAssays
  colIndex <- 1:nrAssays
  
  if(PCAandClust){
    
    ## #############################
    ##
    ## Principal Component Analysis
    ##
    ## #############################

    PCA.in <- M
    
    ## Check if there is any missing data. If so, replace the missing
    ## data with zeros and print a warning message to the user.
    nr.missing<-0

    for(i in 1:nrAssays){
      missing<-which(is.na(PCA.in[,i]))
      PCA.in[missing,i]<-mean(PCA.in[,i],na.rm=TRUE)
      nr.missing <- nr.missing+length(missing)
    }

    if(nr.missing>0){
      warning("pca.R",paste("Missing data at ",nr.missing," positions. This may affect the results of PCA."))
    }
    
    ## Run the PCA algorithm
    pca.res<-prcomp(t(PCA.in), retx = TRUE, center = TRUE, scale. = FALSE, tol = NULL)
    
    ## Create 2D PCA plot
    bitmap(file=paste(picsDir,"pca.png",sep=""), type = "png16m")
    
    pca.title<-"Principal Component Analysis plot"
    plot(pca.res$x[,1], pca.res$x[,2], type = "n", xlab = "First principal component", ylab = "Second principal component", cex.axis = 1.3) 
    text(pca.res$x[,1], pca.res$x[,2], labels=assayNames,  cex=1.5, col = "blue")
    dev.off()

    debug.output("PCA performed")
    
    ## #########################
    ##
    ## Hierarchical Clustering
    ##
    ## #########################
    
    ## Compute distances between arrays
    sampleDists <- as.matrix(dist(t(M), method = clustDistMethod, diag = TRUE, upper = TRUE))

    my.hclust <- function(d){
      hclust(d,method = clustMethod)
    }
    
    ## Create heat map plot
    bitmap(file=paste(picsDir,"clusteredSamples.png", sep=""), type = "png16m")
    
    hv <- heatmap(sampleDists, labRow = assayNames, hclustfun = my.hclust, labCol = assayNames, symm = TRUE)

    rowIndex <- hv$rowInd
    colIndex <- hv$colInd
    
    dev.off()

    debug.output("Hierarchical clustering performed")
  }
  
  ## ##################
  ##
  ## Correlation plots
  ##
  ## ##################

  if(reorder == FALSE){
    rowIndex <- 1:nrAssays
    colIndex <- 1:nrAssays
  }
  
  dpi <- 72
  imgSize <- 6
  
  blankImg <- paste(picsDir,"blank.png",sep ="")
  
  get.blank(fname = blankImg, height = imgSize, width = imgSize, txt = "Click on table elements to change image.")
  
  pairs <- array(NA,0)
  pairPlots <- array(NA,0)   
  r2 <- matrix(NA,nrow = nrAssays, ncol = nrAssays)
  corrPics <- matrix(blankImg,nrow = nrAssays, ncol = nrAssays)
  
  for (i in 1:nrAssays){
    for (j in i:nrAssays){
      if(compMatrix[i,j] == 1){
        print(paste("Computing",i,"-",j,"correlation"))

        tmpPic <- paste(picsDir,"cor",i,"vs",j,".png",sep="")
        
        bitmap(file = tmpPic, type = "png16m", height=imgSize, width=imgSize, res=dpi)
        
        r2[i,j] <- cor(M[,i],M[,j],method = corMethod,use="complete.obs")
        r2[i,j] <- signif(r2[i,j], digits = 3)

        corrPics[i,j] <- tmpPic

        maxVal <- ceiling(max(M[,i],M[,j],na.rm = TRUE))
        minVal <- floor(min(M[,i],M[,j],na.rm = TRUE))
        
        plot(M[,i],M[,j],xlab = realAssayNames[i],ylab = realAssayNames[j],main = paste(assayNames[i]," vs ",assayNames[j]," (",r2[i,j],")", sep = ""), xlim = c(minVal,maxVal), ylim = c(minVal,maxVal), cex.axis = 1.3)

        xtr <- par()$usr
        lines(xtr[1:2],r2[i,j]*xtr[1:2],col = "red")
        
        dev.off()

        ## Store values for symetric comparisons
        if(compMatrix[j,i] == 1){
          r2[j,i] <- r2[i,j]
          corrPics[j,i] <- tmpPic
        }
        debug.output(paste("Computed correlation ",i,"-",j,sep=""))
      }
    }
  }
  
  ## #########################
  ##
  ## Create HTML result files 
  ##
  ## #########################
  
  resFile <- "Results.html"
  
  cat("<html>\n", file=resFile)
  cat(styleStr, file=resFile, append=TRUE)
  
  cat("<h1 align=center>Array Similarity Plots</h1>\n<br>\n", file=resFile, append=TRUE)
  if(PCAandClust){
    ## PCA and Hierarchical clustering plots are put in a table
    cat("<table align=center width=600 border=\"1\" frame=\"border\" rules=\"none\">\n", file=resFile, append=TRUE)
    cat(paste("<tr bgcolor=\"\#ddddff\"><td style=\"text-align:center\">PCA plot (two-dimensinal)</td><td style=\"text-align:center\">Hierarchical clustering (",clustMethod,", ",clustDistMethod,")</td></tr>", sep = ""), file=resFile, append=TRUE)
    cat("<tr><td><img align=center src=\"pics/pca.png\" ></td>", sep="", file=resFile, append=TRUE)
    cat("<td><img align=center src=\"pics/clusteredSamples.png\"></td></tr>\n</table><br><br>\n\n", sep="", file=resFile, append=TRUE)
  }
  
  ## Display table with mapping between array numbers and names
  assayTable <- cbind(assayNames, realAssayNames)
  colnames(assayTable) <- c("ID", "Name")
  cat("<table align=center width=600 border=\"1\" frame=\"border\" rules=\"none\">\n<tr bgcolor=\"ddddff\"><td style=\"text-align:center\">", file=resFile, append=TRUE)
  write.table(assayTable, eol="</tr>\n<tr bgcolor=\"#f0f0ff\">\n<td style=\"text-align:center\">",sep="</td><td style=\"text-align:center\">", file=resFile, row.names=FALSE, na="", quote=FALSE, append=TRUE)
  cat("</td></tr>\n</table><br><br>", file=resFile, append=TRUE)
  
  
  ## Display correlation plots...
  r.seq.from = -1
  r.seq.to = 1
  no.intervals = 200
  
  ## Display scale 
  
  r.seq <- seq(from=r.seq.from, to=r.seq.to, length=no.intervals)
  
  ##  r.col <- rev(heat.colors(no.intervals))
  ## r.col <- rev(rainbow(no.intervals, start=0, end = 2/6))

  r.col <- rgcolors.func(no.intervals)
  
  scaleImg <- paste(picsDir,"scale.png",sep ="")

  bitmap(scaleImg, height = imgSize, width = 1)
  
  image(t(as.matrix(r.seq)), col=r.col, axes = FALSE)
  axis(2, cex.axis = 1.5, labels = seq(r.seq.from,r.seq.to,length = 5), at = seq(0,1,length= 5))

  dev.off()
  
  ## Display correlation table
  
  javaScript <- paste("<script type=\"text/javascript\">\n",
                      "function changeImage(img)\{\n",
                      "document.getElementById('corr_img').",
                      "src=img;\}\n",
                      "</script>");
  
  cat(javaScript,file=resFile, append=TRUE)
  
  cat(paste("<h3 align=center>Pairwise correlations (",corMethod,")</h3>", sep = ""), file=resFile, append=TRUE)
  
  cat("<table align=center border=\"1\" frame=\"border\" rules=\"none\" cellspacing=\"0\" cellpadding=\"0\">", file=resFile, append=TRUE)
  
  ## First table row
  cat("<tr><td style=\"background-color:\#ddddff\">&nbsp;</td>", file=resFile, append=TRUE)
  cat(paste("<td style=\"text-align:center;background-color:\#ddddff\">",colIndex, "</td>"), file=resFile, append=TRUE)
  cat("<td style=\"text-align:center;background-color:\#ddddff\">Correlation plot</td>", file=resFile, append=TRUE)
  cat(paste("<td style=\"text-align:center;background-color:\#ddddff\">Scale</td></tr>\n"), file=resFile, append=TRUE)
  
  ## Function for mapping correlation values to colors
  map.r <- function(val){
    dist <- abs(r.seq - val)
    col.id <- which(dist == min(dist))[1]
    return(r.col[col.id])
  }

  ## Remaining table rows
  
  cellHeight <- ceiling((dpi*imgSize)/nrAssays)
  cellWidth <- cellHeight
  
  for(i in 1:nrAssays){
    cat(paste("<tr><td bgcolor=\"\#ddddff\">",rowIndex[i],"</td>"), file=resFile, append=TRUE)
    for(j in 1:nrAssays){
      r2val <- r2[rowIndex[i], colIndex[j]]
      r2col <- map.r(r2val)
      cell <- paste("<td><a href=\"javascript:changeImage('",corrPics[rowIndex[i],colIndex[j]],"');\" title=\"",r2val,"\"><div style=\"width:",cellWidth,";height:",cellHeight,";background-color:",r2col,";cursor:hand;\">&nbsp;</div></a></td>", sep = "")
      
      cat(cell, file=resFile, append=TRUE)
    }
    
    ## Corralation image and scale goes in the first array row
    if(i == 1){
      cat(paste("<td rowspan=",nrAssays,"><img id=\"corr_img\" src=\"",blankImg,"\"></td>"), file=resFile, append=TRUE)
      cat(paste("<td rowspan=",nrAssays,"><img src=\"",scaleImg,"\"></td>"), file=resFile, append=TRUE)
    }
  
    cat("</tr>\n", file=resFile, append=TRUE)
  }
  
  cat("</table>", file=resFile, append=TRUE)

  if(!continueHTML){
     cat("</body></html>\n", file=resFile, append=TRUE)
  }
  
  return(colIndex)
}
