######################################################################
##
## File: debug.R
## Version: R2:1.0
##
## Author: Adam Ameur, Linnaeus Centre for Bioinformatics, 7/7 2006
##
## Description: Functions for debug handling. Debug is initiated by
##              read.parameters() in common.R depending on the value
##              of the hidden debug argument. Available debug
##              types are:
##
##              type 1 -- Profiling of R code (in R-prof.html)
##              type 2 -- Debug messages stored (in R-debug.txt) 
##              type 3 -- Both profiling and debug performed
##
##              Output is generated if the current debug.level is
##              less than or equal to the hidden level.debug argument.
##              If not set, all debug messages are output.
##
## The process id is written to the file R-pid.txt which is removed
## when all R code has been executed. 
##
#######################################################################


## Global debug variable
DEBUG<-FALSE

## Global profiling variable
PROFILING<-FALSE

## Sets a global variable and starts R profiling
profiling.on<-function(){
  assign("PROFILING",TRUE,.GlobalEnv)
  Rprof()
}

## Writes summary information about profiling (from the file "Rprof.out") to a file
output.profile <- function(fName){
  Rprof(NULL)
  cat("<html><body>", file=fName)
  cat("<table width=600 border=\"1\" frame=\"border\" rules=\"none\" align=\"center\"><tr bgcolor=\"ddddff\"><td>", file=fName, append=TRUE)
  cat("function</td><td>", file=fName, append=TRUE)
  res <- summaryRprof()$by.total
  write.table(res,file=fName, eol="</td><tr bgcolor=\"f0f0ff\">\n<td>",sep="</td><td>",append=TRUE, quote = FALSE)
  cat("</table></body></html>", file=fName, append=TRUE)
  unlink("Rprof.out")
}

## Writes profiling results to the file Rprof.html
write.profiling<-function(){
  if(PROFILING == TRUE){
    output.profile("R-prof.html")
  }
}

## Sets the debug variable, starts a clock and
## stores current time in a global variable 
debug.init<-function(){
  assign("DEBUG",TRUE,.GlobalEnv)
  assign("START_TIME",proc.time(),.GlobalEnv)
  debug.output("Debug initiated")
}

## Sets the debug level for filtering debug output
debug.level.init <- function(debug.level) {
  assign("DEBUG_LEVEL",debug.level,.GlobalEnv)
}

## Checks is the timer has been initialized
debug.is.set<-function(){
  timerVar <- mget("START_TIME", envir=.GlobalEnv, ifnotfound = NA)[[1]]
  if(is.na(timerVar)){
    return(FALSE)
  }
  else{
    return(TRUE)
  }
}

## Checks if debug level has been set
level.is.set <- function() {
  levelVar <- mget("DEBUG_LEVEL", envir=.GlobalEnv, ifnotfound = NA)[[1]]
  if(is.na(levelVar)){
    return(FALSE)
  }
  else{
    return(TRUE)
  }
}

## Checks if current level is less than or equal to global DEBUG_LEVEL
debug.level <- function(level) {
  if (level.is.set()) {
    return(level <= DEBUG_LEVEL)
  }
  return(TRUE)
}

## Outputs the time since first call to system clock to a file (stdout),
## and (optionally) resets the clock variable. The message is only output
## if debug.level is below the global DEBUG_LEVEL.
debug.output<-function(msg, reset = TRUE, file = stdout(), debug.level = 1){
  if(DEBUG == TRUE){
    if(!debug.is.set()){
      die("debug.R", "Timer not initialized!")
    }
    
    if (debug.level(debug.level)) {
      newTime <- proc.time()
      consumedTime <- round(newTime - START_TIME, digits = 2)
      cat("\n%%%%%%%%%%% DEBUG %%%%%%%%%%%%\n", file = file)
      cat(msg, file = file)
      cat(paste("\nTime elapsed: ",consumedTime[3],"s\n", sep =""), file = file)
      cat("%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%\n\n", file = file)
      if(reset == TRUE){
        assign("START_TIME",proc.time(),.GlobalEnv)
      }
    }
  }
}

## Store the pid of the R-process inorder to be able to clean up properly.
debug.store.pid<-function(outfile = "R-pid.txt"){
  cat(paste("Process started:", Sys.time(),"\n"), file = outfile)
  cat(paste("Process id:", Sys.getpid(),"\n"), file = outfile, append=TRUE)
}

## Outputs a number of '<' concatenated to message indicating depth of execution
depth.debug <- function(str,depth,level) {
  res <- ""
  if (depth > 0)
    sapply(1:depth,function(i){res <<- paste(res,"< ")})
  res <- paste(res,str,sep="")
  debug.output(res,debug.level=level)
}
