##############################################################################
##
## %File: heatmap.R
## %Plug-in: Visualization: Heat map
## %Version: R2:1.0
##
## %Author: Adam Ameur, Linnaeus Centre for Bioinformatics
## %Created: 21/10 2004
## %Latest modified: 6/7 2006
##
## %Description: The heat map plug-in is useful for visualizing gene expression
## levels for a few genes. It displays the log2-ratio for each gene throughout
## all arrays in the whole experiment.
##
## %Output: Two plots are produced:
##  - Heat map plot: A matrix with arrays on the x-axis and genes on the y-axis.
##    Each cell is colored from red (high log2-ratio) to green (low log2-ratio).
##    Gene names and ids are displayed when the cursor is moved over a gene.
##  - Line plot: Each gene represented as a line in a coordinate system with
##    arrays on the x-axis and log2-ratio on the y-axis. Useful for visualization
##    of time profiles.
## %Parameters:
##  - Reorder samples in the plot?: If set to <em>yes</em>, the arrays will be
##    ordered according to user input in both plots. Otherwise the arrays will
##    be ordered by their names.
##  - If so, specify the new order: Comma separated string of numbers representing
##    the ordering of arrays. Only used when the parameter above is set.
##
## %Screen shots: heatmap.png (500)
##
#############################################################################

## Ignore warnings
options(warn = -1)

## Load required libraries
library(sma)

## Include functions for BASEfile read/write
source("../../../../configR2.R")
source(paste(config["plugins_R_dir"],"/common.R",sep=""))
source(paste(config["plugins_R_dir"],"/heatmap.inc.R",sep=""))
source(paste(config["plugins_R_dir"],"/rhtml.R",sep=""))

## Read data
pars<-read.parameters()
data<-read.data()

print("Data loaded")

## Store parameters
reorder <- as.integer(pars$reorder)
sampleOrderStr<-pars$sample.order

nrAssays<-length(data)

## Change order on samples in plots if reorder is specified
sampleOrder<-order.sample(reorder, sampleOrderStr, nrAssays)

debug.output("Data and parameters loaded")

## Create an RG object from the loaded data.
## Requires that all arrays have the same layout.
RG<-make.RG.matrix(data, extended = TRUE)

debug.output("RG object created")

## Convert to M-values
M <- log2(RG$R/RG$G)

## #############################################
##
## Create heat map and line plots for all genes
##
## #############################################

heatmap.file<-"heatmap.png"
## Percentage of the total width that the heatmap takes up
heatWidth<-0.35
heightInches<-10
widthInches<-13
bitmap(file=heatmap.file, type = "png16m", height=heightInches, width=widthInches)
layout(matrix(c(1,2),nrow=1,ncol=2),heights=c(5,5),widths=c(heatWidth,1-heatWidth))

## Order cluster according to the sample ordering
ordered.M<-M[,sampleOrder]
ordered.M.centers<-rowMeans(ordered.M)

## Create heat map
zmax<-max(abs(max(M,na.rm=TRUE)),abs(min(M,na.rm=TRUE)))
zmin<-zmax*-1
plot.heatmap(ordered.M, RG$repId, RG$geneSymbol, zmin, zmax)
hh<-heightInches*72
hw<-widthInches*72
texts<-paste("Reporter Id:",RG$repId,", Gene Symbol:", RG$geneSymbol)
heatMapStr<-make.HEAT.imagemap(texts = texts, start = 0, stp = heatWidth*hw, hh = hh, hw=hw)

## Create line plot
ordered.M.centers<-colMeans(ordered.M, na.rm = TRUE)
plot.lines(ordered.M, ordered.M.centers)

dev.off()

debug.output("Heat map created")

## ###########################
##
## Create HTML result page 
##
## ###########################

resFile<-"Results.html"

cat("<html>\n", file=resFile)
cat(styleStr, file=resFile, append=TRUE)

cat("<h1 align=center> Heat map and line plot</h1>", file=resFile, append=TRUE)
cat(heatMapStr, file=resFile, append=TRUE)
cat("<img src = \"heatmap.png\" useMap = \"#map1\" height =\"",hh,"\" width = \"",hw,"\" border = \"0\">",sep="",file  =resFile, append=TRUE)
cat("<p>", file=resFile, append=TRUE)

## Write sample names in a table
orderedAssayNames<-c(RG$assayNames,recursive=TRUE)[sampleOrder]

sampleIds<-c(paste("<td>",1:nrAssays,"</td>", sep=""))
sampleNames<-c(paste("<td>",orderedAssayNames,"</td>", sep=""))
rowStart<-rep("<tr bgcolor=\"f0f0ff\">", nrAssays)

assayData<-data.frame(rowStart,sampleIds,sampleNames)

cat("\n\n<p><table width=\"800\" align=\"center\" cellpadding=\"1\" cellspacing=\"2\" border=\"0\">\n", file=resFile, append=TRUE)
cat("<tr><th class=\"subhead\" colspan=\"1\">Sample number</th><th class=\"subhead\" colspan = \"1\">Sample name</th></tr>\n", file=resFile, append=TRUE)
write.table(assayData,sep="",eol="</tr>\n", file=resFile, col.names=FALSE,row.names=FALSE, na="", quote=FALSE, append=TRUE)
cat("</table>\n", file=resFile, append=TRUE)
cat("<p><hr><p>\n", file=resFile, append=TRUE)

cat("</body></html>", file=resFile, append=TRUE)

debug.output("Finished")

clean.up()
