#######################################################################
##
## File: heatmap.inc.R
## Version: R2:1.0
##
## Author: Adam Ameur, Linnaeus Centre for Bioinformatics, 15/11 2004
## Latest modified: 6/7, 2006
##
## Description: Help functions for visualizing log-ratios of gene lists
##              over multiple arrays in heat maps and line plots. 
##
## These functions are used in heatmap.R and kmeans.R
##
#######################################################################

## Takes M values and plots it as a line plot.
plot.lines <-function(M, M.centers){

  minVal<-floor(min(M, na.rm=TRUE))
  maxVal<-ceiling(max(M, na.rm=TRUE))
  
  nr.assays <- dim(M)[2]
  nr.genes <- dim(M)[1]
  par(mar=c(10,5,10,0))
  plot(c(1,nr.assays),c(minVal,maxVal), type="n", xlab="samples", ylab="M (log ratio)", cex.lab=1.2)
  
  ## Add the genes to the plot
  for(i in 1:nr.genes){
    lines(M[i,])
  }  
  
  if(length(M.centers) > 0){
    lines(M.centers, lwd=2, col="red")
  }
}

## Takes a matrix with M values and produces a heatmap plot.
## Gene names are printed, if the heat map contains few genes
plot.heatmap <- function(M, repIds, geneNames, zmin, zmax){
  
  nr.assays <- dim(M)[2]
  nr.genes <- dim(M)[1]
  
  par(mar=c(0,6,0,0))

  if(nr.genes <= 1){
    plot.new()
    text(0.5,0.5,"Unable to show heat map", cex=0.8, col="red")
    return()
  }
  
  geneLabels<-paste(repIds,geneNames)
  
  plot.mat(M, rlabels=geneLabels, zlim=c(zmin,zmax), nrgcols=100)
}

## Create a new sample ordering from a comma separated string
order.sample<-function(reorder, sampleOrderString, nrAssays){
  sampleOrder<-1:nrAssays

  ## Reorder sample from string
  if(reorder == 1){
    sampleOrder<-as.numeric(strsplit(sampleOrderString,",")[[1]])

    ## Check so that the sample order given by the user is ok
    sortedSampleOrder<-sort(sampleOrder)
    
    if(length(sortedSampleOrder) != nrAssays){
      die("heatmap.inc.R","Wrong number of samples in order string.")
    }
  
    for(i in 1:nrAssays){
      if(sortedSampleOrder[i] != i){
        die("heatmap.inc.R","Wrong sample ids in order string.")
      }
    }
  }

  return(sampleOrder)
}

## Prints the contents of a cluster as a tab delimited text file. 
print.cluster <- function(RG, cluster, cl, cl.nr, ordered.assay.names, out.file){
  
  ## Print header	
  cat("cluster nr: ", cl.nr, ", size: ", cl$size[cl.nr],
      ", withinss: ", cl$withinss[cl.nr], "\n", sep="", file=out.file, append=TRUE)	
  cat("rep","repId","gene",ordered.assay.names, sep="\t", file=out.file, append=TRUE)
  cat("\n", file=out.file, append=TRUE)

  cluster.labels <- matrix(NA, dim(cluster)[1], 3) 

  ## reporter (internal id)
  cluster.labels[,1] <- RG$reporter[cl$cluster == cl.nr]
  ## reporter id
  cluster.labels[,2] <-  RG$repId[cl$cluster == cl.nr]
  ## gene name
  cluster.labels[,3] <- RG$geneSymbol[cl$cluster == cl.nr]

  for(i in 1:dim(cluster)[1]){
    cat(paste(cluster.labels[i,]),sep="\t", file=out.file, append=TRUE)
    cat("\t", file=out.file, append=TRUE)
    cat(paste(cluster[i,]), sep="\t", file=out.file, append=TRUE)
    cat("\n", file=out.file, append=TRUE)
  }
}




