##############################################################################
##
## %File: kmeans.R
## %Plug-in: Clustering: K-means
## %Version: R2:1.0
##
## %Authors: Jakub Orzechowski Westholm, Adam Ameur, Linnaeus Centre for Bioinformatics
## %Created: 20/11 2003
## %Latest modified: 6/7 2006
##
## %Description: This plug-in performs a k-means clustering of the log2-ratios,
##  so that genes with similar expression patterns are grouped together using the
##  algorithm by Hartigan and Wong (1979). The number of clusters (k) is set by
##  the user beforehand. This plug-in doesn't handle missing values in the data,
##  which means that imputation may be required before running it. Note that the
##  k-means clustering is a non-deterministic algorithm, i.e. running the algorithm
##  twice on the same data may give different results.
##
## %Output: A new data set with an extra column named in.cluster is returned.
##  The extra column contains the resulting cluster number for each gene. In addition,
##  two plots are produced for each cluster:
##  - Heat map plot: A matrix with arrays on the x-axis and genes on the y-axis. Each
##    cell is colored from red (high log2-ratio) to green (low log2-ratio). Gene names
##    and ids are displayed when the cursor is moved over a gene.
##  - Line plot: Each gene represented as a line in a coordinate system with arrays on
##    the x-axis and log2-ratio on the y-axis. Useful for visualization of time profiles.
##
## %Parameters:
##  - Number of clusters(K):
##  - Number of iterations:
##  - Reorder samples in the plot?: If set to 'yes', the arrays will be ordered according
##    to user input in both plots. Otherwise the arrays will be ordered by their names.
##  - If so, specify the new order: Comma separated string of numbers representing the
##    ordering of arrays. Only used when the parameter above is set.
##
## %Screen shots: kmeans.png (500)
##
##################################################################################

## Ignore warnings
options(warn = -1)

## Include functions for BASEfile read/write
source("../../../../configR2.R")
source(paste(config["plugins_R_dir"],"/common.R",sep=""))
source(paste(config["plugins_R_dir"],"/heatmap.inc.R",sep=""))
source(paste(config["plugins_R_dir"],"/rhtml.R",sep=""))

## Load required libraries
library(sma)
library(stats)

## Read data
pars<-read.parameters()
data<-read.data()

## Set up directory for the job where all plots are stored
picsDir<-"pics/"
dir.create(picsDir)

## A number specifying the largest number of clusters allowed
MAXNRCLUSTERS<-50

## Store parameters
nr.clusters <- as.integer(pars$nr.clusters)
nr.iterations <- as.integer(pars$nr.iterations)
reorder <- as.integer(pars$reorder)
sampleOrderStr<-pars$sample.order

## Make sure aruments are ok...
if(nr.iterations < 1){
  die("kmeans.R", "Positive nr of iterations expected!") 
}
if(nr.clusters <= 1 || nr.clusters > MAXNRCLUSTERS){     
  die("kmeans.R", paste("Number of clusters must be between 1 and",MAXNRCLUSTERS ,"!"))
}

nrAssays<-length(data)

## Change order on samples in plots if reorder is specified
sampleOrder<-order.sample(reorder, sampleOrderStr, nrAssays)

debug.output("Data and parameters loaded")

## Create an RG object from the loaded data.
## Requires that all arrays have the same layout.
RG<-make.RG.matrix(data, extended = TRUE)

debug.output("RG object created")

## Convert to M-values
M <- log2(RG$R/RG$G)

# Check if there is any missing data. If so, replace the missing
# data with zeros and print a warning message to the user.
if(length(M[is.na(M)])>0){
  nr.missing<-length(M[is.na(M)])
  die("kmeans.R",paste("Missing data at ",nr.missing," positions. Run the imputation first."))
}


## ####################
##
## Do the clustering
##
## ####################

cl <- kmeans(M, nr.clusters, nr.iterations)

debug.output("Clustering done")

## ################################################
##
## Create plots and a text file for each cluster
##
## ################################################ 

outfileNames<-paste(picsDir,"cluster",1:nr.clusters,".png",sep="")
outTextFileNames<-paste(picsDir,"clusterlist",1:nr.clusters,".txt",sep="")

mapnames<-array()
maps<-array()
heatWidth<-0.35
heightInches<-10
widthInches<-13
zmax<-max(abs(max(M,na.rm=TRUE)),abs(min(M,na.rm=TRUE)))
zmin<-zmax*-1
hh<-heightInches*72
hw<-widthInches*72

for (i in 1:nr.clusters){

  nrInCluster<-length(cl$cluster[cl$cluster == i])
  
  if(nrInCluster == 0){
    die("kmeans.R", "Empty cluster!!! Try a smaller value on k!")
  }
  
  ## Set layout for cluster plots
  bitmap(file=outfileNames[i], type = "png16m", height=heightInches, width=widthInches)
  
  layout(matrix(c(1,2),nrow=1,ncol=2),heights=c(5,5),widths=c(heatWidth,1-heatWidth))
  
  ## Extract cluster
  cluster <- M[cl$cluster == i,]

  ## Order cluster according to the sample ordering
  orderedCluster<-NULL
  orderedClusterCenters<-NULL
  
  if(nrInCluster > 1){
    orderedCluster<-cluster[,sampleOrder]
    orderedClusterCenters<-cl$centers[,sampleOrder][i,]
  }
  else{
    orderedCluster<-matrix(cluster[sampleOrder], nrow=1, ncol=nrAssays)
  }

  repIds<-as.character(RG$repId[cl$cluster == i])
  geneSymbols<-as.character(RG$geneSymbol[cl$cluster == i])
  plot.heatmap(orderedCluster, repIds, geneSymbols, zmin, zmax)
    
  orderedAssayNames<-c(RG$assayNames,recursive=TRUE)[sampleOrder]
  
  texts<-paste("Reporter Id: ",repIds,", Gene Symbol:", geneSymbols, sep="")
  mapnames[i]<-paste("map",i,sep="")
  maps[i]<-make.HEAT.imagemap(texts = texts, mapn=mapnames[i], start = 0, stp = heatWidth*hw, hh = hh, hw=hw)
  
  ## Create line plot
  plot.lines(orderedCluster, orderedClusterCenters)

  ## Print cluster as text file
  print.cluster(RG, orderedCluster, cl, i, orderedAssayNames, outTextFileNames[i])

  dev.off()

  debug.output(paste("Produced plots for cluster",i))
}
  
## #################################
##
## Add cluster nr to output data
##
## #################################

for(i in 1:length(data)){
  RG$inCluster <- cl$cluster
}

extraColumn<-c("inCluster")

write.output.from.RG("stdout.txt", RG, extraColumn)

## ###########################
##
## Create HTML result page 
##
## ###########################
resFile<-"Results.html"

blankname<-paste(picsDir,"blank.png", sep="")

get.blank(fname = blankname, width = widthInches, height = heightInches)

heads <- data.frame("Clusters", "Data")
clusterData<-paste("<a href=\"",outTextFileNames,"\">data</a>", sep="")
data <- data.frame(clusterData)
resultTable<-pic.table(header=heads, data = data, pics = outfileNames,width = hw,height = hh, use.names=FALSE,mapnames = mapnames,maps = maps,blankname = blankname)

cat("<html>\n", file=resFile)
cat(styleStr, file=resFile, append=TRUE)

cat("<h1 align=center> Heat maps and line plots for all clusters</h1>", file=resFile, append=TRUE)
cat(resultTable, file=resFile, append = TRUE)

## Write sample names in a table
orderedAssayNames<-c(RG$assayNames,recursive=TRUE)[sampleOrder]

sampleIds<-c(paste("<td>",1:nrAssays,"</td>", sep=""))
sampleNames<-c(paste("<td>",orderedAssayNames,"</td>", sep=""))
rowStart<-rep("<tr bgcolor=\"f0f0ff\">", nrAssays)

assayData<-data.frame(rowStart,sampleIds,sampleNames)

cat("\n\n<p><table width=\"800\" align=\"center\" cellpadding=\"1\" cellspacing=\"2\" border=\"0\">\n", file=resFile, append=TRUE)
cat("<tr><th class=\"subhead\" colspan=\"1\">Sample number</th><th class=\"subhead\" colspan = \"1\">Sample name</th></tr>\n", file=resFile, append=TRUE)
write.table(assayData,sep="",eol="</tr>\n", file=resFile, col.names=FALSE,row.names=FALSE, na="", quote=FALSE, append=TRUE)
cat("</table>\n", file=resFile, append=TRUE)
cat("<p><hr><p>\n", file=resFile, append=TRUE)

cat("</body></html>", file=resFile, append=TRUE)

debug.output("Finished")

clean.up()


