######################################################################
##
## File: layout.R
## Version: R2:1.0
##
## Author: Adam Ameur, Linnaeus Centre for Bioinformatics, 5/7 2006
##
## Description: Functions for handling array grid layouts.
##              Used by visualization and normalization plug-ins.
##
#######################################################################

## Creates a new layout object, either from the information
## in the stdin.txt BASEfile or arguments given by the user. 
create.layout <- function(pars, layout.in, assayId){

  ## Array layout was sent in as an argument to the plug-in
  if(pars$specifyLayout == 1){	
    metaX<-as.integer(pars$ngrid.c.in)
    metaY<-as.integer(pars$ngrid.r.in)
    subX<-as.integer(pars$nspot.c.in)
    subY<-as.integer(pars$nspot.r.in)
  }
  
  ## Layout specified in stdin.txt file
  else{
    if(layout.in$id == assayId){
      metaX<-layout.in$metaX
      metaY<-layout.in$metaY
      subX<-layout.in$subX
      subY<-layout.in$subY
    }
    
    if(layout.in$id != assayId){
      errMsg<-paste("Layout id missing for array ",i,".", sep="")
      die("layout.R", errMsg)
    }
    
    if(metaX == 0 || metaY == 0 || subX == 0 || subY ==0){
      die("layout.R", "Array layout not specified for array ",i,".",sep="")
    }
  }

  ## Set layout
  layout<-list(ngrid.c=metaX,ngrid.r=metaY,nspot.c=subX,nspot.r=subY)
  
  return(layout)
}

## Returns the number of spot positions in a layout
nr.pos.in.layout<-function(layout){

  nrPos<-layout$ngrid.r*layout$ngrid.c*layout$nspot.r*layout$nspot.c

  return(nrPos)
}

## Takes a vector with positions for a set of spots together with the total number 
## of spots on a microarray as input. Returns a vector containing the positions 
## that were not contained in the input vector.
get.na.positions<-function(positionData, nrPos){

  tempPos<-c()

  ## All positions on the array is stored in the array tempPos.
  ## There might be some positions missing if there are NA's at
  ## the last positions.
  tempPos[positionData] <- positionData

  ## Add NA's at the end of the array.
  nrMissing<-nrPos-length(tempPos)

  ## There is something wrong with the array layout if the number
  ## of missing positions is negative
  if(nrMissing < 0){
    die("layout.R:", "Data contains more spots than specified layout.")
  }
  
  naAtEnd<-array(data=NA,dim=nrMissing)
  tempPos<-append(tempPos,naAtEnd)
  
  missPos<-c()
  
  ## The missing positions are the indexes for the NA's in the
  ## tempPos array.
  naPositions<-match(tempPos,NA)
  allPositions<-1:nrPos
  missPos<-naPositions*allPositions
  missPos<-missPos[!is.na(missPos)]
  
  return(missPos)
  
}

## Adds NA to all missing positions in a data set, and returns the
## resulting data set. If the extended argument is set, the function
## expects that input data contains bg and std values in addition to
## flags and intensities.
fill.missing.positions<-function(data, layout, extended = FALSE){
  
  ## The total number of positions on the array
  nrPos<-layout$ngrid.r*layout$ngrid.c*layout$nspot.r*layout$nspot.c

  ## Get a vector with all NA positions in the array
  naPositions<-c()
  naPositions<-get.na.positions(data$position,nrPos) 

  nrMissing<-length(naPositions)
    
  naValues<-array(data=NA, dim=nrMissing)

  ## Append missing positions to position data, and NA to 
  ## all other data fields.
  tempPosition<-append(data$position,naPositions)
  tempIntensity1<-append(data$intensity1,naValues)
  tempIntensity2<-append(data$intensity2,naValues)
  tempFlags<-append(data$flags,naValues)

  ## Sort spots on position and store in assayData
  assayData<-list()

  positionOrder <- rank(tempPosition)
  assayData$intensity1[positionOrder] <- tempIntensity1
  assayData$intensity2[positionOrder] <- tempIntensity2
  assayData$flags[positionOrder] <- tempFlags

  ## Add a number of fields if the extended option is set
  if(extended == TRUE){
    
    tempBCh1Median<-append(data$BCh1Median,naValues)
    tempBCh2Median<-append(data$BCh2Median,naValues)
    tempBCh1SD<-append(data$BCh1SD,naValues)
    tempBCh2SD<-append(data$BCh2SD,naValues)
    
    assayData$BCh1[positionOrder] <- tempBCh1Median
    assayData$BCh2[positionOrder] <- tempBCh2Median
    assayData$BCh1SD[positionOrder] <- tempBCh1SD
    assayData$BCh2SD[positionOrder] <- tempBCh2SD

  }
  
  return(assayData)

}
