################################################################################
##
## %File: merge_assays.R
## %Plug-in: Miscellaneous: Merge assays
## %Version: R2:1.0
##
## %Authors: Jakub Orzechowski Westholm, Adam Ameur, Linnaeus Centre for Bioinformatics
## %Created: 11/12 2004
## %Latest modified: 6/7 2006
##
## %Description: This plug-in merges the spot intensities on two or more replicate
##  arrays by computing the geometric mean of intensity1 and intensity2. Since each
##  resulting value may be a result of several merged spots, all spot-related
##  information (flags, spot diameter, and so on ...) is obsolete in the resulting
##  data set. Therefore, only intensity values and reporter information should be
##  used afterwards!
##
## %Output:  The plug-in creates a new data set where the specified arrays have
##  been merged. A text file is created where all groups of assays that were
##  merged are listed.
##
## %Parameters: The parameters below specify which arrays that are going to be merged.
##  - Assays to merge: A comma separated string of numbers, which corresponds to
##    the arrays that are going to be merged. This option only handles merging one
##    group of arrays and leaves all other ones unchanged.
##  - Merge all assays simultaneously: If selected, all assay names are cut into
##    smaller parts by first removing substrings. After that the suffixes and finally
##    the prefixes are removed. Groups of arrays sharing the same name after this
##    procedure are then merged. Since it may be hard to predict what the 'merge all
##    assays simultaneously' option does, it should not be used unless needed. And
##    look at the resulting text file to make sure it worked.
##  - Substrings to remove: Comma separated strings that are removed from the names
##    when the 'merge all assays simultaneously' option is set. No substrings
##    are removed if left empty.
##  - Prefixes/suffixes to remove: Comma separated strings. The first/last occurrence
##    of the strings together with everything before/after it is removed. Only used
##    when the 'merge all assays simultaneously' option is set. Nothing is removed
##    if left empty.
##
################################################################################

## Ignore warnings
options(warn = -1)

## Include functions for BASEfile read/write
source("../../../../configR2.R")
source(paste(config["plugins_R_dir"],"/common.R",sep=""))

## Load required libraries
library(sma)
library(limma)

############################## Functions ###################################
## First argument is a vector of all assay names, in the same order as 
## in the inout BASE file. The second argument is a vector with the names
## of assays that will be merged. Output is a logical vector, with the 
## same order as thge first argument, with values for each assay: TRUE if
## it will be merged and FALSE otherwise. 
get.merge.vector <- function(all.assays, assays.to.merge){
  merge.vec <- array(FALSE, dim = length(all.assays))

  for(i in 1:length(assays.to.merge)){
    merge.vec[as.integer(assays.to.merge[i])] <- TRUE
  }	
  
  return(merge.vec)
}

## Re order the assays. Returns a vector with a map 
## new.assay.nr -> old.assay.nr given an input list l, where l[i] == TRUE 
## denotes that assay i should be merged and l[i] == FALSE that it should not.
get.order <- function(lst){
  counter <- 1	
  order <- list()
  length(order) <- 1 # make sure that the first element of order exists

  for(i in 1:length(lst)){
    if (lst[[i]] == TRUE){
      order[[1]][length(order[[1]])+1] <- i 
    }
    else{
      counter <- counter+1
      order[[counter]] <- i
    }
  }

  return(order)
}

## Performs a string matching of all assays where given substrings, prefix and suffix are
## removed from all assay names. Returns the resulting names
string.match.all.assays <- function(all.assay.names, string.splits, prefixes, suffixes){
  new.assay.names <- array("",dim=length(data))
  
  for(i in 1:length(data)){
    temp<-all.assay.names[[i]]
    
    ## Remove substrings 
    if(length(string.splits) > 0){
      for(j in 1:length(string.splits)){
        temp<-paste(strsplit(temp,string.splits[[j]])[[1]],collapse="")
      }
    }
    
    ## Remove suffixes
    if(length(suffixes) > 0){
      for(j in 1:length(suffixes)){
        temp<-strsplit(temp,suffixes[[j]])[[1]][1]
      }
    }
    
    ## Remove prefixes
    if(length(prefixes) > 0){
      for(j in 1:length(prefixes)){
        nr.parts.prefix<-length(strsplit(temp,prefixes[[j]])[[1]])
        temp<-strsplit(temp,prefixes[[j]])[[1]][nr.parts.prefix]
      }
    }
    
    print(temp)
    
    new.assay.names[i]<-temp

  }
  
  return(new.assay.names)
}

## Takes a set of assay names that have been string matched as input and returns a
## list with assays that have the same names. The arrays with the same names
## are the ones that will be merged.
get.ordering.for.all.assays<-function(new.assay.names){
  ordering<-list()
  
  j<-1
  already.ordered<-array()
  
  for(i in 1:length(data)){
    if(!(i %in% already.ordered)){
      ordering[[j]]<-which(new.assay.names == new.assay.names[i])
      already.ordered<-c(already.ordered, ordering[[j]])
      j<-j+1
    }
  }

  if(length(ordering) == length(data)){
    die("merge_assays.R", "No merging done since no assays have same name after removing substrings.")
  }
  
  return(ordering)
}

new.assay.names.for.all.assays <- function(temp.assay.names, ordering){
  new.assay.names<-list()
  
  for(i in 1:length(ordering)){
    new.assay.names[[i]]<-temp.assay.names[ordering[[i]][1]]
    print(new.assay.names[[i]])
  }

  return(new.assay.names)
}

## First argument is the intensities (e.g R values), and the second argument
## is the reordering of the assays. The trird argument is a string that is
## used in the names of the columns in the output. Output is an updated version 
## of the intensities where the assays have been merged (by taking the mean) 
## over all  assays, and reorded.
merge.intensities <- function(int,order,tag){
  nr.assays <- length(order)
  nr.reporters <- length(int[,1])
  out.int <- list()
  
  for(i in 1:nr.assays){
    old.assays <- order[[i]]
    ## Here two or more assays are merged
    if(length(old.assays) > 1){
      logs<-log(int[,old.assays])
      out.int[[i]] <- exp(rowMeans(logs,na.rm=TRUE))
      out.int[[i]][which(is.na(out.int[[i]]))]<-NA
    }
    ## Here no assays are merged, so they are just passed along
    else{
      out.int[[i]] <- int[,old.assays]
    }
  }
  
  ## Set column names
  out.int <- data.frame(out.int)
  colnames(out.int)<-paste(tag, "." ,1:dim(out.int)[2], sep="")
  return(out.int)
}

## First argument is the flags, second is the re ordering of the assays. 
## Output is an updated version of the flags table, where the assays have been
## reordered and merged. If several flags F are being merged, the resulting 
## flag will be min(F) 
merge.assaynames <- function(assays, order){
  nr.assays <- length(order)
  out.assays <- list()
 
  for(i in 1:nr.assays){
    old.assays <- order[[i]]
    ## need to 'unlist' for the conversion to character shall work.
    out.assays[[i]] <- paste(unlist(assays[old.assays]),collapse="&")
  }
  
  return(out.assays)
}

################################## main #######################################

pars<-read.parameters()
data<-read.data()

merge.all<-(as.numeric(pars$merge.all.in) == 1)
string.splits<-trimWhiteSpace(strsplit(as.character(pars$substrs.in),",")[[1]])
suffixes<-trimWhiteSpace(strsplit(as.character(pars$suffix.in),",")[[1]])
prefixes<-trimWhiteSpace(strsplit(as.character(pars$prefix.in),",")[[1]])

debug.output("Data and parameters loaded")

## Create an RG object from the loaded data. Only possible if
## all arrays have the same layout.
RG<-make.RG.matrix(data)

debug.output("RG object created")

## Compute the new ordering of assays in the file

all.assay.names <- RG$assayNames

newAssayNames <- list()
ordering <- list()

if(merge.all == TRUE){
  temp.assay.names <- string.match.all.assays(all.assay.names, string.splits, prefixes, suffixes)
  ordering <- get.ordering.for.all.assays(temp.assay.names)
  newAssayNames <- new.assay.names.for.all.assays(temp.assay.names, ordering)
}

if(merge.all == FALSE){
  assay.lst <- strsplit(pars$assays.in, ",")[[1]]

  ## Make sure assays are ok.
  if(is.odd(length(assay.lst)) || length(assay.lst) == 0){
    cat("PLUGIN: mergeassays.R: ", file=stderr())
    cat("Wrong number of assays. Expected an even non-zero number!", file=stderr())
    flush(stderr())
    q(status=-1)
  }

  ordering <- get.order(get.merge.vector(all.assay.names, assay.lst))
  
  newAssayNames <- merge.assaynames(all.assay.names, ordering)
}

print(ordering)

## Write the names of the assays that are merged to a result file
cat("--- Groups of assays that become merged ---\n\n",file="result.txt")

for(i in 1:length(ordering)){
  toBeMerged<-all.assay.names[[ordering[[i]][1]]]
  if(length(ordering[[i]]) > 1){
    for(pos in 2:length(ordering[[i]])){
      toBeMerged<-paste(toBeMerged,", ",all.assay.names[[ordering[[i]][pos]]],sep="")
    }
  }
  #  toBeMerged<-paste(all.assay.names[[ordering[i]]],sep=",")
  cat(paste("Group ",i,":",toBeMerged,"\n\n",sep=""), file="result.txt", append=TRUE)
}

## Merge the intensisites: R, G
new.R <- merge.intensities(RG$R, ordering, "R")
new.G <- merge.intensities(RG$G, ordering, "G")

newAssayIds <- merge.assaynames(RG$assayIds, ordering)

## Build and output new RG
newRG<-list(position=RG$position,reporter=RG$reporter,assayIds=newAssayIds,assayNames=newAssayNames,R=new.R,G=new.G)

## Write output as transformation
write.output.from.RG("stdout.txt", newRG, write.assays.section = TRUE)

debug.output("Finished")

clean.up() 
