##############################################################################
##
## %File: normalization.R
## %Plug-in: Normalization: Within arrays
## %Version: R2:1.0
## 
## %Author: Adam Ameur, Linnaeus Centre for Bioinformatics, 31/3 2004
## %Latest modified: 5/7 2006
##
## %Description: This plug-in normalizes log-ratios for each array individually
##  by various methods for averaging log-ratios to zero within each array or
##  sub-array. The print-tip version of this normalization, which is a local
##  fitting of the regression line, is usually preferred since it corrects for
##  both spatial effects and dye biases. All available methods are listed below:
##  - Median: The median log-ratio is centered on zero.
##  - Loess: All intensities on the array are fitted to a regression line,
##    so that dye biases are removed. Yang et al (2001, 2002)
##  - Print-tip loess: Loess normalization is perfromed within each sub array, so
##    that both spatial effects and dye biases are removed. Yang et al (2001, 2002)
##  - Composite: This method is not yet operational within LCB-DWH.
##  - Robust spline: Robustly fitted regression splines and empirical Bayes shrinkage.
##
## %Output: The plug-in creates a new data set with normalized spot intensities.
##
## %Parameters:
##  - Method: One of median, loess, print-tip loess, composite or robust spline
##  - Specify array layout dimensions: Should be selected only when the print-tip
##    loess method is to be run and array layout dimensions have not been put into
##    the database at an earlier stage. 
##  - Grid rows/columns, Spot rows/columns per print-tip group: Information about
##    the array layout. Must be provided when the 'specify array layout dimensions'
##    option is selected.
##  - Associate quality weights with flag values: Quality weights may be associated
##    with flag values. A quality weight is a number in the interval [0,1] that
##    represents how much a spot is taken into account in the normalization.
##  - Flag values with weight 0, 0.25, 0.5, 0.75: Association between flag values
##    and weights, only used when the associate quality weights with flag values
##    option is selected. For each of the weights, flags values that are to be
##    associated are given as a comma separated string. Spots flagged with values
##    other than the ones specified here get the default weight 1.
##
##############################################################################

## Ignore warnings
options(warn = -1)

source("../../../../configR2.R")

## Include functions for BASEfile read/write
source(paste(config["plugins_R_dir"],"/common.R",sep=""))

## Include help functions for array layout
source(paste(config["plugins_R_dir"],"/layout.R",sep=""))

## Load required libraries
library(limma)

## Load data and parameters
pars<-read.parameters()
data<-read.data()

## Load grid layouts on the arrays. Not required
## for the 'median' and 'loess' methods 
layouts<-read.layout()

if(pars$use.weights == 1){
  
  temp0=strsplit(pars$weight.0,",")
  weights0 = as.integer(temp0[[1]])
  
  temp25=strsplit(pars$weight.25,",")
  weights25 = as.integer(temp25[[1]])	

  temp50=strsplit(pars$weight.50,",")
  weights50 = as.integer(temp50[[1]])  

  temp75=strsplit(pars$weight.75,",")
  weights75 = as.integer(temp75[[1]])

}

nrAssays<-length(data)

## The normalization method is given as one of the strings below.
normMethods<-c("median","loess","printtiploess","composite","robustspline")

## Methods that require layout specific information
layoutMethods<-c("printtiploess","composite","robustspline")

normMethod<-pars$norm.method

if(normMethod == "composite"){
  die("normailzation.R", "Composite method not operational in current version.")
}

if (!(normMethod %in% normMethods)){
  die("normailzation.R", "Normalization method unknown.")
}

debug.output("Data and parameters loaded")

data.transformed<-list()

for (i in 1:nrAssays){

  ## Default settings, used for 'median' and 'loess' methods
  layout <- NULL
  assayData <- data[[i]]
  nrPos <- length(assayData$position)
  
  ## Save all original positions, in case they are altered below
  orig.pos <- data[[i]]$position
  orig.rep <- data[[i]]$reporter
  
  ## Change settings for methods that reqire layout
  if(normMethod %in% layoutMethods){
    
    ## Create array layout
    layout <- create.layout(pars, layouts[[i]], data[[i]]$assayId)
    
    ## The total number of positions on the array
    nrPos<- nr.pos.in.layout(layout)
     
    assayData <- fill.missing.positions(data[[i]], layout)
  }
  
  ## ######################################	
  ##
  ## Create weight vector from user input
  ##
  ## ######################################

  weightArray<-array(data=1,dim=nrPos)

  if(pars$use.weights == 1){

    weightMatrix<-array(data=1,dim=nrPos)
    
    if(length(weights0)>0){
      weightArray0<-vector(length=nrPos, mode="numeric")
      weightArray0<-weightArray0[match(assayData$flags,weights0)]
      weightArray0[!is.na(weightArray0)]<-0
      weightMatrix<-cbind(weightMatrix,weightArray0)
    }

    if(length(weights25)>0){
      weightArray25<-vector(length=nrPos, mode="numeric")
      weightArray25<-weightArray25[match(assayData$flags,weights25)]
      weightArray25[!is.na(weightArray25)]<-0.25
      weightMatrix<-cbind(weightMatrix,weightArray25)
    }

    if(length(weights50)>0){
      weightArray50<-vector(length=nrPos, mode="numeric")
      weightArray50<-weightArray50[match(assayData$flags,weights50)]
      weightArray50[!is.na(weightArray50)]<-0.5
      weightMatrix<-cbind(weightMatrix,weightArray50)
    }

    if(length(weights75)>0){
      weightArray75<-vector(length=nrPos, mode="numeric")
      weightArray75<-weightArray75[match(assayData$flags,weights75)]
      weightArray75[!is.na(weightArray75)]<-0.75
      weightMatrix<-cbind(weightMatrix,weightArray75)
    }
  
    weightArray<-as.matrix(apply(weightMatrix,1,min,na.rm=TRUE))
  }

  ## ###########
  ##
  ## Normalize
  ##		
  ## ###########

  ## Tranform RG intensities to MA
  RG<-list(R=assayData$intensity1, G=assayData$intensity2)

  if(pars$use.weights == 1){
    RG$weights=weightArray
  }

  MA.before <- MA.RG(RG)

  ## Normalize
  MA.after <- normalizeWithinArrays(MA.before,layout,method=normMethod)
	
  ## Inverse transform back to RG from MA
  RG$R<-round((2^(2*MA.after$A+MA.after$M))^0.5,digits=1)
  RG$G<-round((2^(2*MA.after$A-MA.after$M))^0.5,digits=1)

  intensity1 <- RG$R
  intensity2 <- RG$G

  if(normMethod %in% layoutMethods){
    intensity1 <- RG$R[orig.pos]
    intensity2 <- RG$G[orig.pos]
  }

  
  ## #######################################
  ##	
  ## Store results from the normalization. 
  ##
  ## #######################################

  transformed.assay<-list()

  ## Since all positions in RG is ordered on position, 
  ## we can assume that spot with position i is on row i.

  transformed.assay$position<-orig.pos
  transformed.assay$reporter<-orig.rep
  transformed.assay$intensity1<-intensity1
  transformed.assay$intensity2<-intensity2
  transformed.assay$assayId<-data[[i]]$assayId
	
  data.transformed[[i]]<-transformed.assay

  debug.output(paste(normMethod,"normalization performed on array", i))
}

## Write output as transformation
write.output("stdout.txt",data.transformed)

debug.output("Finished")

clean.up()
