##############################################################################
##
## %File: sam.R
## %Plug-in: Hypothesis testing: SAM
## %Version: R2:1.0
##
## %Authors: Jakub Orzechowski Westholm, Adam Ameur, Linnaeus Centre for Bioinformatics
## %Created: 7/1 2004 
## %Latest modified: 25/7 2006
##
## %Description: SAM (Significance Analysis of Microarrays) identifies genes
##  with statistically significant changes in expression by assimilating a set
##  of gene-specific t tests. The SAM method is most often used for detecting
##  differentially expressed genes between two different groups (experimental
##  conditions). Each gene is assigned a score on the basis of its change in
##  gene expression relative to the standard deviation of repeated measurements
##  for that gene. A high SAM delta and abs(SAM score) indicates that the gene
##  is likely to be differentially expressed. See Tusher et al (2001) for more
##  information about the algorithm. This plug-in doesn't handle missing values
##  in the data, which means that imputation may be required before running it.
##
## %Output: The plug-in returns a new data set, with extra columns named SAM.delta
##  and SAM.score. In addition, a number of plots are produced:
##  1. Volcano plot: abs(SAM score) on x-axis vs fold change on y-axis
##  2. SAM plot: Expected vs observed d-values
##  3. 3 x MA-plots: For group1, group2 and group1-group2
##  4. Top table: containing the top differentially expressed genes
##
## %Parameters:
##  - Number of permutations:
##  - Array group 1: Comma separated string with array numbers representing group1. 
##  - Array group 2: Comma separated string with array numbers representing group2.
##    Used only when differences between group1 and group2 are of interest. In case
##    group 2 is empty, a 1-group SAM will be performed on all arrays.
##  - Delta cut-off in plots: All genes above the cut-off will be shown in the plots.
##
## %Screen shots: sam1.png (500), sam2.png (500), sam3.png (400)
##
##############################################################################

## Ignore warnings
options(warn = -1)

## Include functions for BASEfile read/write
source("../../../../configR2.R")
source(paste(config["plugins_R_dir"],"/common.R",sep=""))
source(paste(config["plugins_R_dir"],"/rhtml.R",sep=""))

## Create a file to divert messages created when loading the siggenes library
## The messages would otherwise end up in the plug-in output.
tmpFile <- "tmp_stderr.txt"
tmpStream <- file(tmpFile, open="wt")

## Divert messages to stderr to the temporary file
sink(tmpStream, type="message")

## Load required library. 
library(siggenes)

## Stop sink-ing
sink(type="message")

## Remove temporary file
unlink(tmpFile, recursive = FALSE)

## Load data and parameters
pars<-read.parameters()
data<-read.data()

picsDir<-"pics/"
dir.create(picsDir)

B.value <- as.integer(pars$nr.permutations)
fixed.delta <- as.numeric(pars$fixed.delta)

## ############################
##
## Make sure arguments are ok.
##
## ############################

if(pars$assays1.in == ""){
	die("sam.R", "Assay group 1 empty!")
}

x.vec <- as.numeric(strsplit(pars$assays1.in, ",")[[1]])

## assays2.in might be empty if only one group is used
if(pars$assays2.in == "None" || pars$assays2.in == ""){
  y.vec <- NULL
} else{
  y.vec <- as.numeric(strsplit(pars$assays2.in, ",")[[1]])
}

cl<-array()
cl[x.vec]<-1
cl[y.vec]<-0

## assays1.in and assays2.in must together contain all assay ids
if(length(which(cl == 1)) + length(which(cl == 0)) != length(data)){
  die("sam.R", "Group 1 and 2 must be a partition of all assays!")
}

two.class <- TRUE

if(length(y.vec) == 0){
  two.class <- FALSE
}

debug.output("Data and parameters loaded")

## #########################
##
## Run SAM, store results
##
## #########################

## Create an RG object from the loaded data.
RG<-make.RG.matrix(data, extended=TRUE)

RG$M <- log2(RG$R/RG$G)
RG$A <- log10(RG$R*RG$G)

## Run sam, store score and delta
sam.out <- sam.dstat(RG$M, cl, B=B.value)

sam.score <- sam.out@d
sam.delta <- sam.score
sam.delta[order(sam.delta)] <- abs(sam.delta[order(sam.delta)]-sam.out@d.bar)

## Genes with delta above threshold are considered siginficant
sigIds <- which(sam.delta > fixed.delta)

debug.output("SAM executed")

## ##############
##
## Create plots
##
## ##############

## Calculate average M and A in the two groups
Mgroup1 <- rowMeans(RG$M[,x.vec],na.rm=TRUE)
Agroup1 <- rowMeans(RG$A[,x.vec],na.rm=TRUE)

Mgroup2 <- rep(0,length(Mgroup1))
Agroup2 <- rep(0,length(Agroup1))

if(two.class){
  Mgroup2 <- rowMeans(RG$M[,y.vec],na.rm=TRUE)
  Agroup2 <- rowMeans(RG$A[,y.vec],na.rm=TRUE)
}

FoldChange <- Mgroup1-Mgroup2
A <- rowMeans(RG$A, na.rm = TRUE)

## Plot: Fold change vs. SAM score
scoreImg<-paste(picsDir,"sam_score.png", sep="")

absScores <- abs(sam.score)

xlab <- "M-value"

if(two.class){
  xlab <- "M-value change (group1 - group2)"
}

bitmap(file=scoreImg, type = "png16m")
plot(FoldChange[-sigIds], absScores[-sigIds], xlab = xlab, ylab = "abs(SAM score)", xlim=c(min(FoldChange, na.rm = TRUE), max(FoldChange+1, na.rm = TRUE)), ylim=c(0,max(absScores, na.rm = TRUE)))
points(FoldChange[sigIds], absScores[sigIds], col = "blue", cex = 1.2, font = 2, pch=19)
text(FoldChange[sigIds], absScores[sigIds], labels=RG$geneSymbol[sigIds], col="blue", cex = 1, font = 2, pos = 4)
dev.off()

## Plot: MA, M=fold change for two-class sam 

ylab <- "Average M"

if(two.class){
  ylab <- "Fold change"
}

MAimg12 <- paste(picsDir,"MA_12.png", sep="")
bitmap(file=MAimg12, type = "png16m", height = 6, width = 12)
plot(A, FoldChange, xlab="Average A", ylab=ylab)
text(A[sigIds], FoldChange[sigIds], labels=RG$geneSymbol[sigIds], col="red", cex = 1.2, font = 2)
dev.off()


if(two.class){
  ## Plot: MA, Group1 
  MAimg1 <- paste(picsDir,"MA_1.png", sep="")
  bitmap(file=MAimg1, type = "png16m", height = 6, width = 6)
  plot(Agroup1, Mgroup1, xlab = "Average A", ylab = "Average M")
  text(Agroup1[sigIds], Mgroup1[sigIds], labels=RG$geneSymbol[sigIds], col="red", cex = 1.2, font = 2)
  dev.off()

  ## Plot: MA, Group2 
  MAimg2 <- paste(picsDir,"MA_2.png", sep="")
  bitmap(file=MAimg2, type = "png16m", height = 6, width = 6)
  plot(Agroup2, Mgroup2, xlab = "Average A", ylab = "Average M")
  text(Agroup2[sigIds], Mgroup2[sigIds], labels=RG$geneSymbol[sigIds], col="red", cex=1.2, font = 2)
  dev.off()
}
  
## Plot: SAM plot with different delta lines
deltaImg<-paste(picsDir,"sam_delta.png", sep="")
bitmap(file=deltaImg, type = "png16m")

plot(sam.out, fixed.delta)

dev.off()

debug.output("SAM plots produced")

## #####################################################
## 
## Store SAM results in extra columns and print results
##
## #####################################################

XcolName1<-"SAM.score"
sam.score<-round(sam.score,digits=3)
tmp.cmd<-paste("RG$",XcolName1,"<-sam.score",sep="")
print(tmp.cmd)
eval(parse(text=tmp.cmd))

XcolName2<-"SAM.delta"
sam.delta<-round(sam.delta,digits=3)
tmp.cmd<-paste("RG$",XcolName2,"<-sam.delta",sep="")
print(tmp.cmd)
eval(parse(text=tmp.cmd))

## Write output as transformation
write.output.from.RG("stdout.txt", RG, extraColumns=c(XcolName1, XcolName2))

## ##########################
##
## Print results as html page
##
## ##########################

resFile <- "Results.html"

## Create HTML page with gene table
tableFile <- paste(picsDir, "SAMtable.html", sep = "")

cat("<html>\n", file=tableFile)
cat(styleStr, file=tableFile, append=TRUE)

sigData <- data.frame(cbind(RG$repId[sigIds], RG$geneSymbol[sigIds], sam.delta[sigIds], sam.score[sigIds], round(FoldChange[sigIds], digits = 3)))

sigData <- sigData[order(as.numeric(as.character(sigData[,3])), decreasing = TRUE),]

names(sigData) <- c("ReporterID", "Gene Symbol", "SAM delta", "SAM score (d-value)", "Fold change")

cat(paste("<h3 align=center>Genes with SAM-delta > ",fixed.delta," </h3>",sep=""), file=tableFile, append=TRUE)
cat("<table align=center width=600 border=\"1\" frame=\"border\" rules=\"none\"><tr bgcolor=\"ddddff\"><td>", file=tableFile, append=TRUE)
write.table(sigData, eol="</td><tr bgcolor=\"f0f0ff\">\n<td>",sep="</td><td>", file=tableFile, row.names=FALSE, na="", quote=FALSE, append=TRUE)
cat("</td></tr></table>", file=tableFile, append=TRUE)
cat("</body></html>", file=tableFile, append=TRUE)

## Create html result file 
cat("<html>\n", file=resFile)
cat(styleStr, file=resFile, append=TRUE)

cat("<h1 align=center>Significance Analysis of Microarrays - Results</h1><p>", file=resFile, append=TRUE)
cat(paste("<h3 align=center>All ",length(sigIds)," genes with delta >",fixed.delta," can be viewed <a href=\"pics/SAMtable.html\">here</a></h3><p>"), file=resFile, append=TRUE)

## SAM score and delta plots are put in a table
cat("<table align=center width=600 border=\"1\" frame=\"border\" rules=\"none\">\n", file=resFile, append=TRUE)
cat(paste("<tr bgcolor=\"\#ddddff\"><td style=\"text-align:center\">SAM score vs Fold change</td><td style=\"text-align:center\">SAM plot (for different values of delta)</td></tr>", sep = ""), file=resFile, append=TRUE)
cat(paste("<tr><td><img align=center src=\"",scoreImg,"\" ></td>", sep=""), file=resFile, append=TRUE)
cat(paste("<td><img align=center src=\"",deltaImg,"\"></td></tr>\n</table><br><br>\n\n", sep=""), file=resFile, append=TRUE)


## MA-plots are put in a table
cat("<h3 align=center>MA-plots</h3>", file=resFile, append=TRUE)
cat("<table align=center width=600 border=\"1\" frame=\"border\" rules=\"none\">\n", file=resFile, append=TRUE)
cat("<tr bgcolor=\"\#ddddff\"><td colspan=2 style=\"text-align:center\">MA-plot</td></tr>\n", file=resFile, append=TRUE)
cat(paste("<tr><td colspan=2><img align=center src=\"",MAimg12,"\" ></td></tr>\n", sep=""), file=resFile, append=TRUE)

if(two.class){
cat("<tr bgcolor=\"\#ddddff\"><td style=\"text-align:center\">MA-plot (group1)</td><td style=\"text-align:center\">MA-plot (group2)</td></tr>\n", file=resFile, append=TRUE)
cat(paste("<tr><td><img align=center src=\"",MAimg1,"\" ></td><td><img align=center src=\"",MAimg2,"\"</td></tr>\n", sep=""), file=resFile, append=TRUE)
}

cat("</table><br><br>\n\n", file=resFile, append=TRUE)

cat("</body></html>", file=resFile, append=TRUE)

debug.output("Finished")

clean.up()







