###############################################################################
##
## %File: scaling.R
## %Plug-in: Normalization: Between arrays
## %Version: R2:1.0
##
## %Author: Adam Ameur, Linnaeus Centre for Bioinformatics, 16/4 2004
## %Latest modified: 5/7 2006
##
## %Description: This plug-in normalizes intensities so that the log-ratios have
##  similar distributions across a series of arrays. It might be used when there
##  are substantial differences between the arrays after other normalization methods
##  have been applied. There are a few different methods available:
##  - Scale: Scaling of log-ratios to obtain the same median-abolute-deviation (MAD)
##    across arrays. Yang et al (2001,2002), Smyth and Speed (2003).
##  - Quantile: Ensures that the intensities have the same empirical distribution
##    across arrays and across channels. Yang and Thorne (2003)
##  - Aquantile: Ensures that the A-values (average intensities) have the same
##    empirical distribution across arrays leaving the M-values (log-ratios)
##    unchanged. Yang and Thorne (2003)
##
## %Output: The plug-in creates a new data set with normalized spot intensities.
##
## %Parameters:
## - Method: One of scale, quantile or aquantile
## 
###############################################################################

## Ignore warnings
options(warn = -1)

## Include functions for BASEfile read/write
source("../../../../configR2.R")
source(paste(config["plugins_R_dir"],"/common.R",sep=""))

## Load required libraries
library(limma)

## Load data and parameters 
pars<-read.parameters()
data<-read.data()

## The normalization method is given as one of "scale", "quantile" and "Aquantile"
normMethods<-c("scale","quantile","Aquantile")

normMethod<-pars$norm.method

if (!(normMethod %in% normMethods)){
  die("rnorm_serial.R", "Normalization method unknown.")
}

debug.output("Data and parameters loaded")

## Create an RG object from the loaded data. Only possible if
## all arrays have the same layout.
RG<-make.RG.matrix(data)

debug.output("RG object created")

nrAssays <- length(RG$assayIds)

## Convert RG object to MA
MAbefore <- MA.RG(RG)

## Normalize
MAafter<-normalizeBetweenArrays(MAbefore, method=normMethod)

debug.output("Normalization done")

## Make box-plots before and after normalization
imgFileName=paste("result.png",sep="")

bitmap(file=imgFileName, type = "png16m", height = 12, width = 11, res = 72)
layout(matrix(c(1,2), 2, 1, byrow=TRUE))

boxplot(MAbefore$M~col(MAbefore$M),col=rainbow(nrAssays),main="Before Normalization between arrays")
abline(0,0,col="blue")

boxplot(MAafter$M~col(MAafter$M),col=rainbow(nrAssays),main="After Normalization between arrays")
abline(0,0,col="blue")
dev.off()

debug.output("Plots created")

## Inverse transform back to RG from MA
RG$R=round(2^(2*MAafter$A+MAafter$M))^(0.5)
RG$G=round(2^(2*MAafter$A-MAafter$M))^(0.5)

## Write output as transformation
write.output.from.RG("stdout.txt", RG)

debug.output("Finished")

clean.up()

