#!/usr/local/bin/perl -w

#######################################################################
#
#  File: R.pl
#
#  Creator: Adam Ameur, Linnaeus Centre for Bioinformatics, 9/3 2004
#
#  Description: This script takes a serial BASEfile as input, parses 
#  the file and produces output that can easily be loaded into R. 
#  Finally, an R program specified in the BASEfile program parameters
#  with the id 'fname' is started.
#
#  Input: This program assumes that the BASEfile contains information 
#  about parameters, assays and spots. Information about program 
#  parameters must be stated at the top of the file directly after the 
#  BASEfile line. The sections assays and spots may be situated at 
#  various locations in the file 
#
#  Output: A number of files are created as output
#
#  1) parameters.txt: A tab-delimited file with all program parameters 
#  
#  parameter1 value1
#  parameter2 value2
#  ...
#  parameterN valueN
#
#  2) assays.txt: A tab-delimited file with information about the assays
#
#  assay_id1 assay_name1 assay_annotations1
#  assay_id2 assay_name2 assay_annotations2
#  ...
#  assay_idM assay_nameM assay_annotationsM
#
#  3) layout.txt: A tab-delimited file with information about array and 
#  grid dimensions for each array. Only created if the BASEfile contains 
#  the section layout.
#  
#  assay_id1 metaX   metaY   subX    subY
#  assay_id2 metaX   metaY   subX    subY
#  ...
#  assay_id3 metaX   metaY   subX    subY
#
#
#  4) experiment.txt: A tab-delimited file with information about the 
#  experiment design. Only created if the BASEfile contains the section
#  experiment. (Added 13/8 2004)
#
#
#  5) A file dataXXX.txt is created for each assay. XXX is the assay id.
#  The file is a table with all data for the assay.  
# 
#####################################################################

my $pwdout = `pwd`;
$pwdout =~ /(.*)\/.*\/.*\/.*\/.*?/;
my $root = $1;

push @INC,$root;

require config;

$ENV{'GS_LIB'} = $config::config{"gs_paths"};
$ENV{'PATH'} = '/bin:/usr/bin:/usr/local/bin:/opt/sfw/bin:/usr/sbin:/sbin';

my $R_plug_dir = $config::config{"plugin_R_dir"};


##################
#
# Main program
#
##################

# A global variable that contains names and values for 
# each parameter. The parameters are set when the BASEfile
# is parsed.
my %parlist;

sub parse_basefile;

MAIN:
{
    # Parse the file and set the %parlist variable
    warn "Plug-in starting\nReading BASEfile...\n";
    parse_basefile(STDIN);

    # One of the parameters must have the name "fname".
    # The value of the "fname" parameter is the name
    # of the R program that is going to be executed.
    if(!$parlist{"fname"}){
	die "\'fname\' parameter not specified.";
    }

    warn "Running plug-in: ".$parlist{"fname"}."\n";
    $status = system("R --no-save < $R_plug_dir/$parlist{fname} > r-debug.txt");
    if($status == 0){
	warn "Plug-in completed\n";
    }
    else{
	die "Failed to execute R-script\n";
    }
}

####################################  
#
# Functions for parsing a BASEfile
#
####################################

# This function parses the BASEfile header
sub parse_header{
    my $stream = $_[0];

    my $line = <$stream>;

    if(!($line =~ /BASEfile/ )){
	die "BASEfile header missing.";
    }

}


# This function parses all program parameters and prints them to the file
# parameters.txt. Also sets the global variable %parlist, which is a list
# of all parameters.
sub parse_parameters{
    my $stream = $_[0];

    my $line = <$stream>;

    my @parameters;
    my $arg_index=0;

    while(!($line =~ /\%/)){
    
	if($line =~ /([^\t]+)\t(.*)/){
	    my $arg_name=$1;
	    my $arg_val=$2;
	    $parameters[$arg_index]=$arg_name."\t".$arg_val;
	    $arg_index++;
	    $parlist{$arg_name} = $arg_val;
	}
	else{
	    die "Wrong argument format: ".$line;
	}
    
	$line = <$stream>;
    }

    $line = <$stream>;

    # Print parameters to file
    my $outfile = "parameters.txt";
    open(OUT_FILE, ">".$outfile) or die "Can't open ".$outfile."\n";

    print OUT_FILE "parameter\tvalue\n";

    foreach my $parameter (@parameters){
	print OUT_FILE $parameter."\n";
    }

    close(OUT_FILE);

    print "Parameters OK.\n";
}

# This function parses the section assays, which contains information
# about all assays in the file. This information consists of annotations
# and ids. Information about the assays are printed to the file assays.txt
sub parse_assays{
    my $stream = $_[0];

    my @assays;
    my $number_assays;
    my $assay_columns;

    my $countOK = 0;
    my $columnsOK = 0;
    my $assaysOK = 0;

    my $line = <$stream>;
   
    # Parse assays header
    while(!($line =~ /\%/)){

	if($line =~ /count\s+(\d+)/){
	    $number_assays=$1;

	    $countOK = 1;
	}

	if($line =~ /columns\t([^\t]+)\t([^\t\n]+)(\t*.*)$/){
	    my $id_col = $1;
	    my $name_col = $2;
	    my $other_cols = $3;

	    if (($id_col =~ /id/) and ($name_col =~ /name/)){
		$columnsOK = 1;
	    }
	    else{
		die "Error in column information";
	    }

	    $assay_columns = $id_col."\t".$name_col.$other_cols;
	}

	if($line =~ /annotationColumns\t*(.*)$/){
	    my $annotation_cols = $1;
	}

	$line = <$stream>;
    }

    $line = <$stream>; # Remove %

    my $current_assay_id = 0;

    # Parse assays data.
    while(!($line =~ /^(\s+)$/) && !($line =~ /^section.*/)){
	
	if($line =~ /^([^\t]+)\t(.*)$/){

	    my $annotation_id = $1;
	    my $annotation_others = $2;

	    $assays[$current_assay_id]=$annotation_id."\t".$annotation_others."\n";

	    $current_assay_id++;

	}

	$line = <$stream>;
    }

    if($number_assays == $current_assay_id){
	$assaysOK = 1;
    }
    else{
	die "Error in assays data";
    }

    if($countOK and $columnsOK and $assaysOK){
	print "Section assays OK.\n";
    }
    else{
	die "Error in section assays";
    }

    # Print assays data to file
    my $outfile = "assays.txt";
    open(OUT_FILE, ">".$outfile) or die "Can't open ".$outfile."\n";
    
    print OUT_FILE $assay_columns."\n";

    for my $assay (@assays){
     
	if ($assay =~ /\S+/){
	    print OUT_FILE $assay;
	}
    }

    close(OUT_FILE);
}

# This function parses the section spots for a certain assay and prints 
# the information to the file dataXXX.txt, where XXX is the assay id.
# The table consists of a header and spot data.
sub parse_spots{
    my $stream = $_[0];

    my $current_assay=0;
    my $channels;
    my @reporter_columns;
    my @spot_columns;
    my $assay_id;
    my $current_outfile;

    my $channelsOK;
    my $reporter_columnsOK;
    my $spot_columnsOK;
    my $assay_idOK;
    
    my $line = <$stream>;

    # Parse spots header
    while(!($line =~ /\%/)){
	
	# Extract reporter column names
	if($line =~ /columns\s+(.*)\s+assayData/){
	    my $rep_cols=$1;
	    @reporter_columns = ($rep_cols =~ /(\S+\s*)/g);
	    
	    $reporter_columnsOK = 1;
	}

	if($line =~ /channels\s+(\d+)/){
	    $channels = $1;

	    $channelsOK = 1;
	}

	if($line =~ /assayFields\s+(.+)/){
	    my $spot_cols = $1;
	    @spot_columns = ($spot_cols =~ /(\S+\s*)/g);

	    $spot_columnsOK = 1;
	}

	if($line =~ /assays\s+(\d+)/){
	    $assay_id = $1;

	    $assay_idOK = 1;
	}

	$line = <$stream>;
    }

    if ($reporter_columnsOK and $spot_columnsOK and  $assay_idOK){
	print "Section spots OK.\n";
    }
    else{
	die "Error in section spots"
    }

    # Open stream to temporary file, where data for this assay is 
    # printed as a table.

    my $outfile = "data".$assay_id.".txt";
    open(OUT_FILE, ">".$outfile) or die "Can't open ".$outfile."\n";

    # Print table header
    for my $rep_col (@reporter_columns){
	print OUT_FILE $rep_col;
    }

    if(@spot_columns > 0){
	print OUT_FILE "\t";
    }

    for my $spot_col (@spot_columns){
        print OUT_FILE $spot_col;
    }

    print OUT_FILE "\n";

    # Parse spots data and print to file
    while(!($line =~ /^(\s+)$/)){
	$line = <$stream>;
	
	print OUT_FILE $line;
    }

    close(OUT_FILE);
}

# This function parses the section layout, which contains information about
# how print tip groups are organized on the array.
sub parse_layout{
    my $stream = $_[0];
    
    my $line = <$stream>;

    my $outfile = "layout.txt";
    open(OUT_FILE, ">".$outfile) or die "Can't open ".$outfile."\n";

    my $column_ids;

    # Parse layout header
    while(!($line =~ /\%/)){
	$line = <$stream>;

	if($line =~ /columns\s+(.+)\s*/){
	    $column_ids = $1;
	}
    }

    print OUT_FILE $column_ids."\n";

    # Parse layout data
    while(!($line =~ /^(\s+)$/)){
	$line = <$stream>;
	print OUT_FILE $line;
    }

    close(OUT_FILE);
}


# This function parses the section experiment, which contains information about
# the experiment design. Added 13/8 2004.
sub parse_experiment{
    my $stream = $_[0];
    
    my $line = <$stream>;

    my $outfile = "experiment.txt";
    open(OUT_FILE, ">".$outfile) or die "Can't open ".$outfile."\n";

    my $column_ids;

    # Parse experiment header
    while(!($line =~ /\%/)){
	$line = <$stream>;
	if($line =~ /columns\s+(.+)\s*/){
	    $column_ids = $1;
	}
    }

    print OUT_FILE $column_ids."\n";

    # Parse experiment data
    while(!($line =~ /^(\s+)$/)){
	$line = <$stream>;
	print OUT_FILE $line;
    }

    close(OUT_FILE);
}



# Function for parsing a whole serial BASEfile
sub parse_basefile{
     my $stream = $_[0];

     # Process BASEfile header
     parse_header($stream);

     # Process program parameters
     parse_parameters($stream);

     # Process assay and spot data
     while(!eof($stream)){
	 my $line = <$stream>;
    
	 print "Parsing: ".$line;

	 if($line =~ /section\s+(\S+)/){
	     $section_type = $1;
	     if($section_type =~ /assays/){
		 parse_assays($stream);
	     }
	     if($section_type =~ /spots/){
		 parse_spots($stream);
	     }
	     if($section_type =~ /layout/){
		 parse_layout($stream);
	     }
	     if($section_type =~ /experiment/){
		 parse_experiment($stream);
	     }
	 }
	 else{
	     die "Error in BASEfile: ".$line;
	 }
     }
     
     print "BASEfile OK\n";

}




